@@ -42,7 +42,50 @@
 extern "C" {
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+GS_EXPORT NSString *const NSCalendarIdentifierGregorian;
+GS_EXPORT NSString *const NSCalendarIdentifierBuddhist;
+GS_EXPORT NSString *const NSCalendarIdentifierChinese;
+GS_EXPORT NSString *const NSCalendarIdentifierCoptic;
+GS_EXPORT NSString *const NSCalendarIdentifierEthiopicAmeteMihret;
+GS_EXPORT NSString *const NSCalendarIdentifierEthiopicAmeteAlem;
+GS_EXPORT NSString *const NSCalendarIdentifierHebrew;
+GS_EXPORT NSString *const NSCalendarIdentifierISO8601;
+GS_EXPORT NSString *const NSCalendarIdentifierIndian;
+GS_EXPORT NSString *const NSCalendarIdentifierIslamic;
+GS_EXPORT NSString *const NSCalendarIdentifierIslamicCivil;
+GS_EXPORT NSString *const NSCalendarIdentifierJapanese;
+GS_EXPORT NSString *const NSCalendarIdentifierPersian;
+GS_EXPORT NSString *const NSCalendarIdentifierRepublicOfChina;
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_10, GS_API_LATEST)
+GS_EXPORT NSString *const NSCalendarIdentifierIslamicTabular;
+GS_EXPORT NSString *const NSCalendarIdentifierIslamicUmmAlQura;
+#endif
+
+// NSCalendarOptions enum
+// These values are currently NOT supported in this NSCalendar implementation.
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9, GS_API_LATEST)
+typedef NSUInteger NSCalendarOptions;
+enum
+{
+  NSCalendarWrapComponents = (1UL << 0),
+
+  NSCalendarMatchStrictly = (1ULL << 1),
+  NSCalendarSearchBackwards = (1ULL << 2),
+
+  NSCalendarMatchPreviousTimePreservingSmallerUnits = (1ULL << 8),
+  NSCalendarMatchNextTimePreservingSmallerUnits = (1ULL << 9),
+  NSCalendarMatchNextTime = (1ULL << 10),
+
+  NSCalendarMatchFirst = (1ULL << 12),
+  NSCalendarMatchLast = (1ULL << 13)
+};
+#endif
+
 typedef NSUInteger NSCalendarUnit;
+
+/* Old-style NSCalendarUnit declarations, deprecated */
 enum
 {
   NSEraCalendarUnit = (1UL << 1),
@@ -65,6 +108,33 @@ enum
 #endif
 };
 
+/* New-style NSCalendarUnit declarations */
+enum
+{
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9, GS_API_LATEST)
+  NSCalendarUnitEra = (1UL << 1),
+  NSCalendarUnitYear = (1UL << 2),
+  NSCalendarUnitMonth = (1UL << 3),
+  NSCalendarUnitDay = (1UL << 4),
+  NSCalendarUnitHour = (1UL << 5),
+  NSCalendarUnitMinute = (1UL << 6),
+  NSCalendarUnitSecond = (1UL << 7),
+  NSCalendarUnitWeekday = (1UL << 9),
+  NSCalendarUnitWeekdayOrdinal = (1UL << 10),
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+  NSCalendarUnitQuarter = (1UL << 11),
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7, GS_API_LATEST)
+  NSCalendarUnitWeekOfMonth = (1UL << 12),
+  NSCalendarUnitWeekOfYear = (1UL << 13),
+  NSCalendarUnitYearForWeekOfYear = (1UL << 14),
+  NSCalendarUnitNanosecond = (1 << 15), // FIXME: unimplemented
+  NSCalendarUnitCalendar = (1 << 20), // FIXME: unimplemented
+  NSCalendarUnitTimeZone = (1 << 21) // FIXME: unimplemented
+#endif
+};
+
 enum
 {
   NSWrapCalendarComponents = (1UL << 0)
