@@ -191,6 +191,8 @@ extern "C" {
 @class NSError;
 @class NSURL;
 
+@protocol NSFileManagerDelegate;
+
 /* MacOS-X defines OSType as a 32bit unsigned integer.
  */
 #ifndef OSTYPE_DECLARED
@@ -210,6 +212,7 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
 {
 #if	GS_EXPOSE(NSFileManager)
 @private
+  id<NSFileManagerDelegate> _delegate;
   NSString	*_lastError;
 #endif
 #if     GS_NONFRAGILE
@@ -229,6 +232,15 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
  */
 + (NSFileManager*) defaultManager;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+#if GS_HAS_DECLARED_PROPERTIES
+@property (assign) id<NSFileManagerDelegate> delegate;
+#else
+- (id<NSFileManagerDelegate>) delegate;
+- (void) setDelegate: (id<NSFileManagerDelegate>)delegate;
+#endif
+#endif
+
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 - (NSDictionary *) attributesOfItemAtPath: (NSString*)path
 				    error: (NSError**)error;
@@ -290,6 +302,14 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
  */
 - (BOOL) removeItemAtURL: (NSURL*)url
                    error: (NSError**)error;
+/**
+ * Creates a symbolic link at the path
+ * that point to the destination path.<br />
+ * Returns YES on success, otherwise NO.
+ */
+- (BOOL) createSymbolicLinkAtPath: (NSString*)path
+              withDestinationPath: (NSString*)destPath
+                            error: (NSError**)error;
 #endif
 
 /**
@@ -333,6 +353,9 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
  * it can't be read for some reason).
  */
 - (NSArray*) contentsOfDirectoryAtPath: (NSString*)path error: (NSError**)error;
+
+- (NSDictionary*) attributesOfFileSystemForPath: (NSString*)path
+                                          error: (NSError**)error;
 #endif
 
 - (BOOL) copyPath: (NSString*)source
@@ -444,6 +467,7 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
  * if they wish to deal with copy and move operations performed
  * by NSFileManager.
  */
+
 @interface NSObject (NSFileManagerHandler)
 /**
  * <p>When an error occurs during a copy or move operation, the file manager
@@ -495,7 +519,6 @@ typedef NSUInteger NSDirectoryEnumerationOptions;
      willProcessPath: (NSString*)path;
 @end
 
-
 /**
  *  <p>This is a subclass of <code>NSEnumerator</code> which provides a full
  *  listing of all the files beneath a directory and its subdirectories.
@@ -659,6 +682,77 @@ GS_EXPORT NSString* const NSFileSystemFreeNodes;
 - (NSUInteger) fileSystemFileNumber;
 @end
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
+
+@protocol NSFileManagerDelegate <NSObject>
+#if GS_PROTOCOLS_HAVE_OPTIONAL
+@optional
+#else
+@end
+@interface NSObject (NSFileManagerDelegate)
+#endif
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldCopyItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldCopyItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        copyingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         copyingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldMoveItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldMoveItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         movingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+          movingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldLinkItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldLinkItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        linkingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         linkingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldRemoveItemAtPath: (NSString *)path;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldRemoveItemAtURL: (NSURL *)URL;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+       removingItemAtPath: (NSString *)path;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        removingItemAtURL: (NSURL *)URL;
+@end
+
+#endif
+
 #if	defined(__cplusplus)
 }
 #endif
