@@ -78,10 +78,12 @@ extern "C" {
 #endif
 }
 
-/**
- *  Try to acquire lock and return immediately, YES if succeeded, NO if not.
+#if !NO_GNUSTEP
+/** Report whether this lock is held by the current thread.<br />
+ * Raises an exception if this is not supported by the system lock mechanism.
  */
-- (BOOL) tryLock;
+- (BOOL) isLockedByCurrentThread;
+#endif
 
 /**
  *  Try to acquire lock and return before limit, YES if succeeded, NO if not.
@@ -93,12 +95,17 @@ extern "C" {
  */
 - (void) lock;
 
+/**
+ *  Try to acquire lock and return immediately, YES if succeeded, NO if not.
+ */
+- (BOOL) tryLock;
+
 /**
  *  Relinquish lock.
  */
 - (void) unlock;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 /** Return the name of the receiver or nil of none has been set.
  */
 - (NSString*) name;
@@ -183,6 +190,13 @@ extern "C" {
  */
 - (NSInteger) condition;
 
+#if !NO_GNUSTEP
+/** Report whether this lock is held by the current thread.<br />
+ * Raises an exception if this is not supported by the system lock mechanism.
+ */
+- (BOOL) isLockedByCurrentThread;
+#endif
+
 /*
  * Acquiring and releasing the lock.
  */
@@ -236,7 +250,7 @@ extern "C" {
  */
 - (void) unlock;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 /** Return the name of the receiver or nil of none has been set.
  */
 - (NSString*) name;
@@ -265,6 +279,13 @@ extern "C" {
 #endif
 }
 
+#if !NO_GNUSTEP
+/** Report whether this lock is held by the current thread.<br />
+ * Raises an exception if this is not supported by the system lock mechanism.
+ */
+- (BOOL) isLockedByCurrentThread;
+#endif
+
 /**
  *  Try to acquire lock regardless of condition and return immediately, YES if
  *  succeeded, NO if not.
@@ -286,7 +307,7 @@ extern "C" {
  */
 - (void) unlock;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 /** Return the name of the receiver or nil of none has been set.
  */
 - (NSString*) name;
