@@ -36,6 +36,16 @@
 extern "C" {
 #endif
 
+@class  NSThread;
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
+enum {
+  NSTaskTerminationReasonExit = 1,
+  NSTaskTerminationReasonUncaughtSignal = 2
+};
+typedef NSInteger NSTaskTerminationReason;
+#endif
+
 @interface NSTask : NSObject
 {
 #if	GS_EXPOSE(NSTask)
@@ -53,9 +63,17 @@ extern "C" {
   BOOL		_hasTerminated;
   BOOL		_hasCollected;
   BOOL		_hasNotified;
+  NSThread      *_launchingThread;
+  NSTaskTerminationReason       _terminationReason;
 #endif
-#if	!GS_NONFRAGILE
-  void		*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
@@ -91,6 +109,9 @@ extern "C" {
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (int) processIdentifier;
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
+- (NSTaskTerminationReason) terminationReason;
+#endif
 - (int) terminationStatus;
 
 /*
