@@ -26,7 +26,7 @@
 #define __NSURLDownload_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -46,8 +46,9 @@ extern "C" {
  */
 @interface NSURLDownload : NSObject
 {
-@private
+#if	GS_EXPOSE(NSURLDownload)
   void *_NSURLDownloadInternal;
+#endif
 }
 
 /**
@@ -118,7 +119,18 @@ extern "C" {
 /**
  * Protocol for delegate used to report the progress of the download.
  */
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7, GS_API_LATEST)
+@protocol NSURLDownloadDelegate <NSObject>
+#if GS_PROTOCOLS_HAVE_OPTIONAL
+@optional
+#else
+@end
 @interface NSObject (NSURLDownloadDelegate)
+#endif
+#else
+@interface NSObject (NSURLDownloadDelegate)
+#endif
 
 /**
  * Called immediately once the download has started.
