@@ -26,7 +26,7 @@
 #define __NSURLProtectionSpace_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -41,7 +41,7 @@ extern NSString * const NSURLProtectionSpaceHTTPProxy;	/** An HTTP proxy */
 extern NSString * const NSURLProtectionSpaceHTTPSProxy;	/** An HTTPS proxy */
 extern NSString * const NSURLProtectionSpaceSOCKSProxy;	/** A SOCKS proxy */
 
-/** Default authentication */
+/** Default authentication (Basic) */
 extern NSString * const NSURLAuthenticationMethodDefault;
 
 /** HTML form authentication */
@@ -53,6 +53,15 @@ extern NSString * const NSURLAuthenticationMethodHTTPBasic;
 /** HTTP Digest authentication */
 extern NSString * const NSURLAuthenticationMethodHTTPDigest;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+extern NSString * const NSURLAuthenticationMethodNTLM;
+extern NSString * const NSURLAuthenticationMethodNegotiate;
+#endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+extern NSString * const NSURLAuthenticationMethodClientCertificate;
+extern NSString * const NSURLAuthenticationMethodServerTrust;
+#endif
 
 /**
  * Class to encapsulate a protection space ... where authentication is
@@ -129,6 +138,14 @@ authenticationMethod: (NSString *)authenticationMethod;
  */
 - (BOOL) receivesCredentialSecurely;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+#if GS_HAS_DECLARED_PROPERTIES
+@property (readonly, copy) NSArray *distinguishedNames;
+#else
+- (NSArray *) distinguishedNames;
+#endif
+#endif
+
 @end
 
 #if	defined(__cplusplus)
