@@ -143,6 +143,7 @@ static GSLazyLock *local_lock = nil;
 typedef	unsigned char	unc;
 static NSStringEncoding	defEnc = GSUndefinedEncoding;
 static NSStringEncoding	natEnc = GSUndefinedEncoding;
+static NSStringEncoding	icuEnc = GSUndefinedEncoding;
 static NSStringEncoding *_availableEncodings = 0;
 
 struct _strenc_ {
@@ -153,7 +154,7 @@ struct _strenc_ {
 					 * iconv perform conversions to/from
 					 * this encoding.
 					 * NB. do not put a null pointer in this
-					 * field in the table, use "" instread.
+					 * field in the table, use "" instead.
 					 */
   BOOL			eightBit;	/* Flag to say whether this encoding
 					 * can be stored in a byte array ...
@@ -192,7 +193,7 @@ static struct _strenc_ str_encoding_table[] = {
   {NSSymbolStringEncoding,
     "NSSymbolStringEncoding","",0,0,0},
   {NSNonLossyASCIIStringEncoding,
-    "NSNonLossyASCIIStringEncoding","",1,1,0},
+    "NSNonLossyASCIIStringEncoding","",0,1,0},
   {NSShiftJISStringEncoding,
     "NSShiftJISStringEncoding","SHIFT-JIS",0,0,0},
   {NSISOLatin2StringEncoding,
@@ -714,6 +715,12 @@ uni_is_decomp(unichar u)
     }
 }
 
+static inline int
+octdigit(int c)
+{
+  return (c >= '0' && c < '8');
+}
+
 /**
  * Function to check a block of data for validity as a unicode string and
  * say whether it contains solely ASCII or solely Latin1 data.<br />
@@ -745,11 +752,6 @@ GSUnicode(const unichar *chars, unsigned length,
 		  while (i < length)
 		    {
 		      c = chars[i++];
-		      if (c == 0xfffe || c == 0xffff
-			|| (c >= 0xfdd0 && c <= 0xfdef))
-			{
-			  return i - 1;	// Non-characters.
-			}
 		      if (c >= 0xdc00 && c <= 0xdfff)
 		        {
 			  return i - 1;	// Second half of a surrogate pair.
@@ -776,62 +778,6 @@ GSUnicode(const unichar *chars, unsigned length,
   return i;
 }
 
-#if	GS_WITH_GC
-
-#define	GROW() \
-if (dst == 0) \
-  { \
-    /* \
-     * Data is just being discarded anyway, so we can \
-     * reset the offset into the local buffer on the \
-     * stack and pretend the buffer has grown. \
-     */ \
-    ptr = buf - dpos; \
-    bsize = dpos + BUFSIZ; \
-    if (extra != 0) \
-      { \
-	bsize--; \
-      } \
-  } \
-else if (zone == 0) \
-  { \
-    result = NO; /* No buffer growth possible ... fail. */ \
-    goto done; \
-  } \
-else \
-  { \
-    unsigned	grow = slen; \
-\
-    if (grow < bsize + BUFSIZ) \
-      { \
-	grow = bsize + BUFSIZ; \
-      } \
-    grow *= sizeof(unichar); \
-\
-    if (ptr == buf || ptr == *dst) \
-      { \
-	unichar	*tmp; \
-\
-	tmp = NSAllocateCollectable(grow + extra, 0); \
-	if (tmp != 0) \
-	  { \
-	    memcpy(tmp, ptr, bsize * sizeof(unichar)); \
-	  } \
-	ptr = tmp; \
-      } \
-    else \
-      { \
-	ptr = NSReallocateCollectable(ptr, grow + extra, 0); \
-      } \
-    if (ptr == 0) \
-      { \
-	return NO;	/* Not enough memory */ \
-      } \
-    bsize = grow / sizeof(unichar); \
-  }
-
-#else	/* GS_WITH_GC */
-
 #define	GROW() \
 if (dst == 0) \
   { \
@@ -884,8 +830,6 @@ else \
     bsize = grow / sizeof(unichar); \
   }
 
-#endif	/* GS_WITH_GC */
-
 /**
  * Function to convert from 8-bit data to 16-bit unicode characters.
  * <p>The dst argument is a pointer to a pointer to a buffer in which the
@@ -1030,16 +974,6 @@ GSToUnicode(unichar **dst, unsigned int *size, const unsigned char *src,
 	          u = u & ~(0xffffffff << ((5 * sle) + 1));
 		  spos += sle;
 
-		  /*
-		   * We discard invalid codepoints here.
-		   */
-		  if (u > 0x10ffff || u == 0xfffe || u == 0xffff
-		    || (u >= 0xfdd0 && u <= 0xfdef))
-		    {
-		      result = NO;	// Invalid character.
-		      goto done;
-		    }
-
 		  if ((u >= 0xd800) && (u <= 0xdfff))
 		    {
 		      result = NO;	// Unmatched half of surrogate pair.
@@ -1084,6 +1018,146 @@ GSToUnicode(unichar **dst, unsigned int *size, const unsigned char *src,
 	break;
 
       case NSNonLossyASCIIStringEncoding:
+        {
+          unsigned int  index = 0;
+          unsigned int  count = 0;
+
+          while (index < slen)
+            {
+              uint8_t	c = (uint8_t)((unc)src[index++]);
+
+              if ('\\' == c)
+                {
+                  if (index < slen)
+                    {
+                      c = (uint8_t)((unc)src[index++]);
+                      if ('\\' == c)
+                        {
+                          count++;      // Escaped backslash
+                        }
+                      else if (octdigit(c)
+                        && (index < slen && octdigit(src[index++]))
+                        && (index < slen && octdigit(src[index++])))
+                        {
+                          count++;  // Octal escape
+                        }
+                      else if (('u' == c)
+                        && (index < slen && isxdigit(src[index++]))
+                        && (index < slen && isxdigit(src[index++]))
+                        && (index < slen && isxdigit(src[index++]))
+                        && (index < slen && isxdigit(src[index++])))
+                        {
+                          count++;      // Hex escape for unicode
+                        }
+                      else
+                        {
+                          result = NO;	// illegal backslash escape
+                          goto done;
+                        }
+                    }
+                  else
+                    {
+                      result = NO;	// unbalanced backslash
+                      goto done;
+                    }
+                }
+              else
+                {
+                  count++;
+                }
+            }
+          
+          if (dst == 0)
+            {
+              /* Just counting bytes.
+               */
+              dpos += count;
+            }
+          else
+            {
+              if (dpos + count + (extra ? 1 : 0) > bsize)
+                {
+                  if (zone == 0)
+                    {
+                      result = NO; /* No buffer growth possible ... fail. */
+                      goto done;
+                    }
+                  else
+                    {
+                      unsigned	grow = (dpos + count) * sizeof(unichar);
+                      unichar	*tmp;
+
+                      tmp = NSZoneMalloc(zone, grow + extra * sizeof(unichar));
+                      if ((ptr == buf || ptr == *dst) && (tmp != 0))
+                        {
+                          memcpy(tmp, ptr, bsize * sizeof(unichar));
+                        }
+                      if (ptr != buf && ptr != *dst)
+                        {
+                          NSZoneFree(zone, ptr);
+                        }
+                      ptr = tmp;
+                      if (ptr == 0)
+                        {
+                          return NO;	/* Not enough memory */
+                        }
+                      bsize = grow / sizeof(unichar);
+                    }
+                }
+              while (spos < slen)
+                {
+                  uint8_t	c = (uint8_t)((unc)src[spos++]);
+
+                  if ('\\' == c)
+                    {
+                      c = (uint8_t)((unc)src[spos++]);
+                      if ('\\' == c)
+                        {
+                          ptr[dpos++] = c;
+                        }
+                      else if ('u' == c)
+                        {
+                          int   i = 0;
+
+                          for (count = 0; count < 4; count++)
+                            {
+                              c = (uint8_t)((unc)src[spos++]);
+                              i *= 16;
+                              if (isdigit(c))
+                                {
+                                  i += c - '0';
+                                }
+                              else if (isupper(c))
+                                {
+                                  i += 10 + c - 'A';
+                                }
+                              else
+                                {
+                                  i += 10 + c - 'a';
+                                }
+                            }
+                          ptr[dpos++] = i;
+                        }
+                      else
+                        {
+                          int   i = c - '0';
+
+                          c = (uint8_t)((unc)src[spos++]);
+                          i = i * 8 + c - '0';
+                          c = (uint8_t)((unc)src[spos++]);
+                          i = i * 8 + c - '0';
+                          ptr[dpos++] = i;
+                        }
+                    }
+                  else
+                    {
+                      ptr[dpos++] = c;
+                    }
+                }
+            }
+          }
+	break;
+
       case NSASCIIStringEncoding:
 	if (dst == 0)
 	  {
@@ -1094,7 +1168,7 @@ GSToUnicode(unichar **dst, unsigned int *size, const unsigned char *src,
 	  }
         else
 	  {
-	    /* Because we know that each ascii chartacter is exactly
+	    /* Because we know that each ascii character is exactly
 	     * one unicode character, we can check the destination
 	     * buffer size and allocate more space in one go, before
 	     * entering the loop where we deal with each character.
@@ -1111,21 +1185,15 @@ GSToUnicode(unichar **dst, unsigned int *size, const unsigned char *src,
 		    unsigned	grow = (dpos + slen) * sizeof(unichar);
 		    unichar	*tmp;
 
-#if	GS_WITH_GC
-		    tmp = NSAllocateCollectable(grow + extra, 0);
-#else
-		    tmp = NSZoneMalloc(zone, grow + extra);
-#endif
+		    tmp = NSZoneMalloc(zone, grow + extra * sizeof(unichar));
 		    if ((ptr == buf || ptr == *dst) && (tmp != 0))
 		      {
 			memcpy(tmp, ptr, bsize * sizeof(unichar));
 		      }
-#if	!GS_WITH_GC
 		    if (ptr != buf && ptr != *dst)
 		      {
 			NSZoneFree(zone, ptr);
 		      }
-#endif
 		    ptr = tmp;
 		    if (ptr == 0)
 		      {
@@ -1175,21 +1243,15 @@ GSToUnicode(unichar **dst, unsigned int *size, const unsigned char *src,
 		    unsigned	grow = (dpos + slen) * sizeof(unichar);
 		    unichar	*tmp;
 
-#if	GS_WITH_GC
-		    tmp = NSAllocateCollectable(grow + extra, 0);
-#else
-		    tmp = NSZoneMalloc(zone, grow + extra);
-#endif
+		    tmp = NSZoneMalloc(zone, grow + extra * sizeof(unichar));
 		    if ((ptr == buf || ptr == *dst) && (tmp != 0))
 		      {
 			memcpy(tmp, ptr, bsize * sizeof(unichar));
 		      }
-#if	!GS_WITH_GC
 		    if (ptr != buf && ptr != *dst)
 		      {
 			NSZoneFree(zone, ptr);
 		      }
-#endif
 		    ptr = tmp;
 		    if (ptr == 0)
 		      {
@@ -1264,21 +1326,15 @@ tables:
 		    unsigned	grow = (dpos + slen) * sizeof(unichar);
 		    unichar	*tmp;
 
-#if	GS_WITH_GC
-		    tmp = NSAllocateCollectable(grow + extra, 0);
-#else
-		    tmp = NSZoneMalloc(zone, grow + extra);
-#endif
+		    tmp = NSZoneMalloc(zone, grow + extra * sizeof(unichar));
 		    if ((ptr == buf || ptr == *dst) && (tmp != 0))
 		      {
 			memcpy(tmp, ptr, bsize * sizeof(unichar));
 		      }
-#if	!GS_WITH_GC
 		    if (ptr != buf && ptr != *dst)
 		      {
 			NSZoneFree(zone, ptr);
 		      }
-#endif
 		    ptr = tmp;
 		    if (ptr == 0)
 		      {
@@ -1436,17 +1492,12 @@ done:
 	  /*
 	   * Temporary string was requested ... make one.
 	   */
-#if	GS_WITH_GC
-	  r = NSAllocateCollectable(bytes, 0);
-	  memcpy(r, ptr, bytes);
-#else
 	  r = GSAutoreleasedBuffer(bytes);
 	  memcpy(r, ptr, bytes);
 	  if (ptr != buf && ptr != *dst)
 	    {
 	      NSZoneFree(zone, ptr);
 	    }
-#endif
 	  ptr = r;
 	  *dst = ptr;
 	}
@@ -1462,11 +1513,7 @@ done:
 	    {
 	      unichar	*tmp;
 
-#if	GS_WITH_GC
-	      tmp = NSAllocateCollectable(bytes, 0);
-#else
 	      tmp = NSZoneMalloc(zone, bytes);
-#endif
 	      if (tmp != 0)
 		{
 		  memcpy(tmp, ptr, bytes);
@@ -1475,11 +1522,7 @@ done:
 	    }
 	  else
 	    {
-#if	GS_WITH_GC
-	      ptr = NSReallocateCollectable(ptr, bytes, 0);
-#else
 	      ptr = NSZoneRealloc(zone, ptr, bytes);
-#endif
 	    }
 	  *dst = ptr;
 	}
@@ -1493,12 +1536,10 @@ done:
 	  *dst = ptr;
 	}
     }
-#if	!GS_WITH_GC
   else if (ptr != buf && dst != 0 && ptr != *dst)
     {
       NSZoneFree(zone, ptr);
     }
-#endif
 
   if (dst)
     NSCAssert(*dst != buf, @"attempted to pass out pointer to internal buffer");
@@ -1508,62 +1549,6 @@ done:
 
 #undef	GROW
 
-
-#if	GS_WITH_GC 
-
-#define	GROW() \
-if (dst == 0) \
-  { \
-    /* \
-     * Data is just being discarded anyway, so we can \
-     * reset the offset into the local buffer on the \
-     * stack and pretend the buffer has grown. \
-     */ \
-    ptr = buf - dpos; \
-    bsize = dpos + BUFSIZ; \
-    if (extra != 0) \
-      { \
-	bsize--; \
-      } \
-  } \
-else if (zone == 0) \
-  { \
-    result = NO; /* No buffer growth possible ... fail. */ \
-    goto done; \
-  } \
-else \
-  { \
-    unsigned	grow = slen; \
-\
-    if (grow < bsize + BUFSIZ) \
-      { \
-	grow = bsize + BUFSIZ; \
-      } \
-\
-    if (ptr == buf || ptr == *dst) \
-      { \
-	unsigned char	*tmp; \
-\
-	tmp = NSAllocateCollectable(grow + extra, 0); \
-	if (tmp != 0) \
-	  { \
-	    memcpy(tmp, ptr, bsize); \
-	  } \
-	ptr = tmp; \
-      } \
-    else \
-      { \
-	ptr = NSReallocateCollectable(ptr, grow + extra, 0); \
-      } \
-    if (ptr == 0) \
-      { \
-	return NO;	/* Not enough memory */ \
-      } \
-    bsize = grow; \
-  }
-
-#else	/* GS_WITH_GC */
-
 #define	GROW() \
 if (dst == 0) \
   { \
@@ -1615,8 +1600,6 @@ else \
     bsize = grow; \
   }
 
-#endif	/* GS_WITH_GC */
-
 static inline int chop(unichar c, _ucc_ *table, int hi)
 {
   int	lo = 0;
@@ -1812,10 +1795,7 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
 		    }
 
 		  // 0xfeff is a zero-width-no-break-space inside text
-		  if (u1 == 0xfffe			// unexpected BOM
-		    || u1 == 0xffff			// not a character
-		    || (u1 >= 0xfdd0 && u1 <= 0xfdef)	// invalid character
-		    || (u1 >= 0xdc00 && u1 <= 0xdfff))	// bad pairing
+		  if (u1 >= 0xdc00 && u1 <= 0xdfff)	// bad pairing
 		    {
 		      if (strict)
 			{
@@ -1934,10 +1914,7 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
 		    }
 
 		  // 0xfeff is a zero-width-no-break-space inside text
-		  if (u1 == 0xfffe			// unexpected BOM
-		    || u1 == 0xffff			// not a character
-		    || (u1 >= 0xfdd0 && u1 <= 0xfdef)	// invalid character
-		    || (u1 >= 0xdc00 && u1 <= 0xdfff))	// bad pairing
+		  if (u1 >= 0xdc00 && u1 <= 0xdfff)	// bad pairing
 		    {
 		      if (strict)
 			{
@@ -2032,6 +2009,137 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
         break;
 
       case NSNonLossyASCIIStringEncoding:
+        {
+          unsigned int  index = 0;
+          unsigned int  count = 0;
+
+          if (YES == swapped)
+            {
+              while (index < slen)
+                {
+                  unichar	u = src[index++];
+
+                  u = (((u & 0xff00) >> 8) + ((u & 0x00ff) << 8));
+                  if (u < 256)
+                    {
+                      if ((u >= ' ' && u < 127)
+                        || '\r' == u || '\n' == u || '\t' == u)
+                        {
+                          count++;
+                          if ('\\' == u)
+                            {
+                              count++;
+                            }
+                        }
+                      else
+                        {
+                          count += 4;
+                        }
+                    }
+                  else
+                    {
+                      count += 12;
+                    }
+                }
+            }
+          else
+            {
+              while (index < slen)
+                {
+                  unichar	u = src[index++];
+
+                  if (u < 256)
+                    {
+                      if ((u >= ' ' && u < 127)
+                        || '\r' == u || '\n' == u || '\t' == u)
+                        {
+                          count++;
+                          if ('\\' == u)
+                            {
+                              count++;
+                            }
+                        }
+                      else
+                        {
+                          count += 4;
+                        }
+                    }
+                  else
+                    {
+                      count += 6;
+                    }
+                }
+            }
+          if (dst == 0)
+            {
+              /* Just counting bytes ...
+               */
+              dpos = count;
+            }
+          else
+            {
+              /* We can now check the destination buffer size and allocate
+               * more space in one go, before entering the loop where we
+               * deal with each character.
+               */
+              if (count > bsize)
+                {
+                  if (zone == 0)
+                    {
+                      result = NO; /* No buffer growth possible ... fail. */
+                      goto done;
+                    }
+                  else
+                    {
+                      uint8_t	*tmp;
+
+                      tmp = NSZoneMalloc(zone, count + extra);
+                      if (ptr != buf && ptr != *dst)
+                        {
+                          NSZoneFree(zone, ptr);
+                        }
+                      ptr = tmp;
+                      if (ptr == 0)
+                        {
+                          return NO;	/* Not enough memory */
+                        }
+                      bsize = count;
+                    }
+                }
+              index = 0;
+              while (index < slen)
+                {
+                  unichar	u = src[index++];
+
+                  if (YES == swapped)
+                    {
+                      u = (((u & 0xff00) >> 8) + ((u & 0x00ff) << 8));
+                    }
+                  if (u < 256)
+                    {
+                      if ((u >= ' ' && u < 127)
+                        || '\r' == u || '\n' == u || '\t' == u)
+                        {
+                          ptr[dpos++] = (unsigned char)u;
+                          if ('\\' == u)
+                            {
+                              ptr[dpos++] = (unsigned char)u;
+                            }
+                        }
+                      else
+                        {
+                          dpos += sprintf((char*)&ptr[dpos], "\\%03o", u);
+                        }
+                    }
+                  else
+                    {
+                      dpos += sprintf((char*)&ptr[dpos], "\\u%04x", u);
+                    }
+                }
+            }
+          }
+        goto done;
+
       case NSASCIIStringEncoding:
 	base = 128;
 	goto bases;
@@ -2051,7 +2159,7 @@ bases:
 	  }
         else
 	  {
-	    /* Because we know that each ascii chartacter is exactly
+	    /* Because we know that each ascii character is exactly
 	     * one unicode character, we can check the destination
 	     * buffer size and allocate more space in one go, before
 	     * entering the loop where we deal with each character.
@@ -2067,15 +2175,11 @@ bases:
 		  {
 		    uint8_t	*tmp;
 
-#if	GS_WITH_GC
-		    tmp = NSAllocateCollectable(slen + extra, 0);
-#else
 		    tmp = NSZoneMalloc(zone, slen + extra);
 		    if (ptr != buf && ptr != *dst)
 		      {
 			NSZoneFree(zone, ptr);
 		      }
-#endif
 		    ptr = tmp;
 		    if (ptr == 0)
 		      {
@@ -2432,17 +2536,12 @@ iconv_start:
 	  /*
 	   * Temporary string was requested ... make one.
 	   */
-#if	GS_WITH_GC
-	  r = NSAllocateCollectable(bytes, 0);
-	  memcpy(r, ptr, bytes);
-#else
 	  r = GSAutoreleasedBuffer(bytes);
 	  memcpy(r, ptr, bytes);
 	  if (ptr != buf && ptr != *dst)
 	    {
 	      NSZoneFree(zone, ptr);
 	    }
-#endif
 	  ptr = r;
 	  *dst = ptr;
 	}
@@ -2458,11 +2557,7 @@ iconv_start:
 	    {
 	      unsigned char	*tmp;
 
-#if	GS_WITH_GC
-	      tmp = NSAllocateCollectable(bytes, 0);
-#else
 	      tmp = NSZoneMalloc(zone, bytes);
-#endif
 	      if (tmp != 0)
 		{
 		  memcpy(tmp, ptr, bytes);
@@ -2471,11 +2566,7 @@ iconv_start:
 	    }
 	  else
 	    {
-#if	GS_WITH_GC
-	      ptr = NSReallocateCollectable(ptr, bytes, 0);
-#else
 	      ptr = NSZoneRealloc(zone, ptr, bytes);
-#endif
 	    }
 	  *dst = ptr;
 	}
@@ -2489,12 +2580,10 @@ iconv_start:
 	  *dst = ptr;
 	}
     }
-#if	!GS_WITH_GC
   else if (ptr != buf && dst != 0 && ptr != *dst)
     {
       NSZoneFree(zone, ptr);
     }
-#endif
 
   if (dst)
     NSCAssert(*dst != buf, @"attempted to pass out pointer to internal buffer");
@@ -2543,15 +2632,134 @@ GSPrivateAvailableEncodings()
   return _availableEncodings;
 }
 
+NSStringEncoding
+GSPrivateCStringEncoding(const char *encoding)
+{
+  NSStringEncoding  enc = GSUndefinedEncoding;
+
+  if (0 != encoding)
+    {
+      GSSetupEncodingTable();
+
+      if (strcmp(encoding, "ANSI_X3.4-1968") == 0 /* glibc */
+        || strcmp(encoding, "ISO_646.IRV:1983") == 0 /* glibc */
+        || strcmp(encoding, "646") == 0 /* Solaris NetBSD */)
+        enc = NSISOLatin1StringEncoding;
+      else if (strcmp(encoding, "EUC-JP") == 0 /* glibc */
+        /* HP-UX IRIX OSF/1 Solaris NetBSD */
+        || strcmp(encoding, "eucJP") == 0
+        || strcmp(encoding, "IBM-eucJP") == 0 /* AIX */)
+        enc = NSJapaneseEUCStringEncoding;
+      else if (strcmp(encoding, "UTF-8") == 0 /* glibc AIX OSF/1 Solaris */
+        || strcmp(encoding, "utf8") == 0 /* HP-UX */)
+        enc = NSUTF8StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-1") == 0 /* glibc */
+        /* AIX IRIX OSF/1 Solaris NetBSD */
+        || strcmp(encoding, "ISO8859-1") == 0
+        || strcmp(encoding, "iso88591") == 0 /* HP-UX */)
+        enc = NSISOLatin1StringEncoding;
+      else if (strcmp(encoding, "IBM-932") == 0 /* AIX */
+        || strcmp(encoding, "SJIS") == 0 /* HP-UX OSF/1 NetBSD */
+        || strcmp(encoding, "PCK") == 0 /* Solaris */)
+        enc = NSShiftJISStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-2") == 0 /* glibc */
+        /* AIX IRIX OSF/1 Solaris NetBSD */
+        || strcmp(encoding, "ISO8859-2") == 0
+        || strcmp(encoding, "iso88592") == 0 /* HP-UX */)
+        enc = NSISOLatin2StringEncoding;
+      else if (strcmp(encoding, "CP1251") == 0 /* glibc */
+        || strcmp(encoding, "ansi-1251") == 0 /* Solaris */)
+        enc = NSWindowsCP1251StringEncoding;
+      else if (strcmp(encoding, "CP1252") == 0 /*  */
+        || strcmp(encoding, "IBM-1252") == 0 /* AIX */)
+        enc = NSWindowsCP1252StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-5") == 0 /* glibc */
+        /* AIX IRIX OSF/1 Solaris NetBSD */
+        || strcmp(encoding, "ISO8859-5") == 0
+        || strcmp(encoding, "iso88595") == 0 /* HP-UX */)
+        enc = NSISOCyrillicStringEncoding;
+      else if (strcmp(encoding, "KOI8-R") == 0 /* glibc */
+        || strcmp(encoding, "koi8-r") == 0 /* Solaris */)
+        enc = NSKOI8RStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-3") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-3") == 0 /* Solaris */)
+        enc = NSISOLatin3StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-4") == 0 /*  */
+        || strcmp(encoding, "ISO8859-4") == 0 /* OSF/1 Solaris NetBSD */)
+        enc = NSISOLatin4StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-6") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-6") == 0 /* AIX Solaris */
+        || strcmp(encoding, "iso88596") == 0 /* HP-UX */)
+        enc = NSISOArabicStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-7") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-7") == 0 /* AIX IRIX OSF/1 Solaris */
+        || strcmp(encoding, "iso88597") == 0 /* HP-UX */)
+        enc = NSISOGreekStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-8") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-8") == 0 /* AIX OSF/1 Solaris */
+        || strcmp(encoding, "iso88598") == 0 /* HP-UX */)
+        enc = NSISOHebrewStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-9") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-9") == 0 /* AIX IRIX OSF/1 Solaris */
+        || strcmp(encoding, "iso88599") == 0 /* HP-UX */)
+        enc = NSISOLatin5StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-10") == 0 /*  */
+        || strcmp(encoding, "ISO8859-10") == 0 /*  */)
+        enc = NSISOLatin6StringEncoding;
+      else if (strcmp(encoding, "TIS-620") == 0 /* glibc AIX */
+        || strcmp(encoding, "tis620") == 0 /* HP-UX */
+        || strcmp(encoding, "TIS620.2533") == 0 /* Solaris */
+        || strcmp(encoding, "TACTIS") == 0 /* OSF/1 */)
+        enc = NSISOThaiStringEncoding;
+      else if (strcmp(encoding, "ISO-8859-13") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-13") == 0 /*  */
+        || strcmp(encoding, "IBM-921") == 0 /* AIX */)
+        enc = NSISOLatin7StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-14") == 0 /* glibc */
+        || strcmp(encoding, "ISO8859-14") == 0 /*  */)
+        enc = NSISOLatin8StringEncoding;
+      else if (strcmp(encoding, "ISO-8859-15") == 0 /* glibc */
+        /* AIX OSF/1 Solaris NetBSD */
+        || strcmp(encoding, "ISO8859-15") == 0
+        || strcmp(encoding, "iso885915") == 0 /* HP-UX */)
+        enc = NSISOLatin9StringEncoding;
+      else if (strcmp(encoding, "GB2312") == 0 /* glibc */
+        || strcmp(encoding, "gb2312") == 0 /* Solaris */
+        || strcmp(encoding, "eucCN") == 0 /* IRIX NetBSD */
+        || strcmp(encoding, "IBM-eucCN") == 0 /* AIX */
+        || strcmp(encoding, "hp15CN") == 0 /* HP-UX */)
+        enc = NSGB2312StringEncoding;
+      else if (strcmp(encoding, "BIG5") == 0 /* glibc Solaris NetBSD */
+        || strcmp(encoding, "big5") == 0 /* AIX HP-UX OSF/1 */)
+        enc = NSBIG5StringEncoding;
+      else if (strcmp(encoding, "EUC-KR") == 0 /* glibc */
+        || strcmp(encoding, "eucKR") == 0 /* HP-UX IRIX OSF/1 NetBSD */
+        || strcmp(encoding, "IBM-eucKR") == 0 /* AIX */
+        || strcmp(encoding, "5601") == 0 /* Solaris */)
+        enc = NSKoreanEUCStringEncoding;
+    }
+
+  if (enc == GSUndefinedEncoding)
+    {
+      enc = NSISOLatin1StringEncoding;
+    }
+  else if (GSPrivateIsEncodingSupported(enc) == NO)
+    {
+      fprintf(stderr, "WARNING: %s - encoding not implemented as "
+              "c string encoding.\n", encoding);
+      fprintf(stderr, "  NSISOLatin1StringEncoding used instead.\n");
+      enc = NSISOLatin1StringEncoding;
+    }
+
+  return enc;
+}
+
 NSStringEncoding
 GSPrivateDefaultCStringEncoding()
 {
   if (defEnc == GSUndefinedEncoding)
     {
       const char	*encoding = 0;
-#if HAVE_LANGINFO_CODESET
-      char		encbuf[BUFSIZ];
-#endif
       unsigned int	count;
 
       GSSetupEncodingTable();
@@ -2563,150 +2771,6 @@ GSPrivateDefaultCStringEncoding()
 	  return defEnc;
 	}
 
-      if (natEnc == GSUndefinedEncoding)
-	{
-	  /* Encoding not set */
-#if HAVE_UNICODE_UCNV_H
-
-          const char *defaultName;
-          UErrorCode err = U_ZERO_ERROR;
-
-          defaultName = ucnv_getDefaultName ();
-          encoding = ucnv_getStandardName (defaultName, "MIME", &err);
-          if (0 == encoding)
-            {
-              encoding = ucnv_getStandardName (defaultName, "IANA", &err);
-            }
-#endif
-#if HAVE_LANGINFO_CODESET
-          if (0 == encoding)
-            {
-              char  *old;
-              /* Take it from the system locale information.  */
-              [gnustep_global_lock lock];
-              /* Initialise locale system by setting current locale from
-               * environment and then resetting it.  Must be done before
-               * any call to nl_langinfo()
-               */
-              if (0 != (old = setlocale(LC_CTYPE, "")))
-                {
-                  setlocale(LC_CTYPE, old);
-                }
-              strncpy(encbuf, nl_langinfo(CODESET), sizeof(encbuf)-1);
-              encbuf[sizeof(encbuf)-1] = '\0';
-              [gnustep_global_lock unlock];
-              encoding = encbuf;
-            }
-#endif
-
-          if (0 != encoding)
-            {
-              /*
-               * First handle the fallback response from nl_langinfo() ...
-               * if we are getting the default value we can't assume that
-               * the user has set anything up at all, so we must use the
-               * OpenStep/GNUstep default encoding ... latin1, even though
-               * the nl_langinfo() stuff would say default is ascii.
-               */
-              if (strcmp(encoding, "ANSI_X3.4-1968") == 0 /* glibc */
-                || strcmp(encoding, "ISO_646.IRV:1983") == 0 /* glibc */
-                || strcmp(encoding, "646") == 0 /* Solaris NetBSD */)
-                natEnc = NSISOLatin1StringEncoding;
-              else if (strcmp(encoding, "EUC-JP") == 0 /* glibc */
-                /* HP-UX IRIX OSF/1 Solaris NetBSD */
-                || strcmp(encoding, "eucJP") == 0
-                || strcmp(encoding, "IBM-eucJP") == 0 /* AIX */)
-                natEnc = NSJapaneseEUCStringEncoding;
-              else if (strcmp(encoding, "UTF-8") == 0 /* glibc AIX OSF/1 Solaris */
-                || strcmp(encoding, "utf8") == 0 /* HP-UX */)
-                natEnc = NSUTF8StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-1") == 0 /* glibc */
-                /* AIX IRIX OSF/1 Solaris NetBSD */
-                || strcmp(encoding, "ISO8859-1") == 0
-                || strcmp(encoding, "iso88591") == 0 /* HP-UX */)
-                natEnc = NSISOLatin1StringEncoding;
-              else if (strcmp(encoding, "IBM-932") == 0 /* AIX */
-                || strcmp(encoding, "SJIS") == 0 /* HP-UX OSF/1 NetBSD */
-                || strcmp(encoding, "PCK") == 0 /* Solaris */)
-                natEnc = NSShiftJISStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-2") == 0 /* glibc */
-                /* AIX IRIX OSF/1 Solaris NetBSD */
-                || strcmp(encoding, "ISO8859-2") == 0
-                || strcmp(encoding, "iso88592") == 0 /* HP-UX */)
-                natEnc = NSISOLatin2StringEncoding;
-              else if (strcmp(encoding, "CP1251") == 0 /* glibc */
-                || strcmp(encoding, "ansi-1251") == 0 /* Solaris */)
-                natEnc = NSWindowsCP1251StringEncoding;
-              else if (strcmp(encoding, "CP1252") == 0 /*  */
-                || strcmp(encoding, "IBM-1252") == 0 /* AIX */)
-                natEnc = NSWindowsCP1252StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-5") == 0 /* glibc */
-                /* AIX IRIX OSF/1 Solaris NetBSD */
-                || strcmp(encoding, "ISO8859-5") == 0
-                || strcmp(encoding, "iso88595") == 0 /* HP-UX */)
-                natEnc = NSISOCyrillicStringEncoding;
-              else if (strcmp(encoding, "KOI8-R") == 0 /* glibc */
-                || strcmp(encoding, "koi8-r") == 0 /* Solaris */)
-                natEnc = NSKOI8RStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-3") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-3") == 0 /* Solaris */)
-                natEnc = NSISOLatin3StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-4") == 0 /*  */
-                || strcmp(encoding, "ISO8859-4") == 0 /* OSF/1 Solaris NetBSD */)
-                natEnc = NSISOLatin4StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-6") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-6") == 0 /* AIX Solaris */
-                || strcmp(encoding, "iso88596") == 0 /* HP-UX */)
-                natEnc = NSISOArabicStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-7") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-7") == 0 /* AIX IRIX OSF/1 Solaris */
-                || strcmp(encoding, "iso88597") == 0 /* HP-UX */)
-                natEnc = NSISOGreekStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-8") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-8") == 0 /* AIX OSF/1 Solaris */
-                || strcmp(encoding, "iso88598") == 0 /* HP-UX */)
-                natEnc = NSISOHebrewStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-9") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-9") == 0 /* AIX IRIX OSF/1 Solaris */
-                || strcmp(encoding, "iso88599") == 0 /* HP-UX */)
-                natEnc = NSISOLatin5StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-10") == 0 /*  */
-                || strcmp(encoding, "ISO8859-10") == 0 /*  */)
-                natEnc = NSISOLatin6StringEncoding;
-              else if (strcmp(encoding, "TIS-620") == 0 /* glibc AIX */
-                || strcmp(encoding, "tis620") == 0 /* HP-UX */
-                || strcmp(encoding, "TIS620.2533") == 0 /* Solaris */
-                || strcmp(encoding, "TACTIS") == 0 /* OSF/1 */)
-                natEnc = NSISOThaiStringEncoding;
-              else if (strcmp(encoding, "ISO-8859-13") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-13") == 0 /*  */
-                || strcmp(encoding, "IBM-921") == 0 /* AIX */)
-                natEnc = NSISOLatin7StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-14") == 0 /* glibc */
-                || strcmp(encoding, "ISO8859-14") == 0 /*  */)
-                natEnc = NSISOLatin8StringEncoding;
-              else if (strcmp(encoding, "ISO-8859-15") == 0 /* glibc */
-                /* AIX OSF/1 Solaris NetBSD */
-                || strcmp(encoding, "ISO8859-15") == 0
-                || strcmp(encoding, "iso885915") == 0 /* HP-UX */)
-                natEnc = NSISOLatin9StringEncoding;
-              else if (strcmp(encoding, "GB2312") == 0 /* glibc */
-                || strcmp(encoding, "gb2312") == 0 /* Solaris */
-                || strcmp(encoding, "eucCN") == 0 /* IRIX NetBSD */
-                || strcmp(encoding, "IBM-eucCN") == 0 /* AIX */
-                || strcmp(encoding, "hp15CN") == 0 /* HP-UX */)
-                natEnc = NSGB2312StringEncoding;
-              else if (strcmp(encoding, "BIG5") == 0 /* glibc Solaris NetBSD */
-                || strcmp(encoding, "big5") == 0 /* AIX HP-UX OSF/1 */)
-                natEnc = NSBIG5StringEncoding;
-              else if (strcmp(encoding, "EUC-KR") == 0 /* glibc */
-                || strcmp(encoding, "eucKR") == 0 /* HP-UX IRIX OSF/1 NetBSD */
-                || strcmp(encoding, "IBM-eucKR") == 0 /* AIX */
-                || strcmp(encoding, "5601") == 0 /* Solaris */)
-                natEnc = NSKoreanEUCStringEncoding;
-            }
-        }
-
       encoding = getenv("GNUSTEP_STRING_ENCODING");
       if (encoding != 0)
 	{
@@ -2732,7 +2796,7 @@ GSPrivateDefaultCStringEncoding()
 	}
       if (defEnc == GSUndefinedEncoding)
 	{
-	  defEnc = natEnc;
+	  defEnc = GSPrivateICUCStringEncoding();
 	}
       if (defEnc == GSUndefinedEncoding)
 	{
@@ -2747,11 +2811,6 @@ GSPrivateDefaultCStringEncoding()
 	  defEnc = NSISOLatin1StringEncoding;
 	}
 
-      if (natEnc == GSUndefinedEncoding)
-        {
-	  natEnc = defEnc;
-	}
-
       [local_lock unlock];
     }
   return defEnc;
@@ -2781,15 +2840,63 @@ GSPrivateIsByteEncoding(NSStringEncoding encoding)
   return encInfo->eightBit;
 }
 
+/* Returns the C-String encoding as used by native locale functions.
+ * We can use this to convert strings produced by those functions to
+ * NSString objects.
+ */
 NSStringEncoding
 GSPrivateNativeCStringEncoding()
 {
   if (natEnc == GSUndefinedEncoding)
     {
-      /* GSPrivateDefaultCStringEncoding() will actually set the encoding.
+      char	encbuf[BUFSIZ];
+#if HAVE_LANGINFO_CODESET
+      char      *old;
+
+      /* Take it from the system locale information.  */
+      [gnustep_global_lock lock];
+      /* Initialise locale system by setting current locale from
+       * environment and then resetting it.  Must be done before
+       * any call to nl_langinfo()
        */
-      GSPrivateDefaultCStringEncoding();
+      if (0 != (old = setlocale(LC_CTYPE, "")))
+        {
+          setlocale(LC_CTYPE, old);
+        }
+      strncpy(encbuf, nl_langinfo(CODESET), sizeof(encbuf)-1);
+      encbuf[sizeof(encbuf)-1] = '\0';
+      [gnustep_global_lock unlock];
+#else
+      encbuf[0] = '\0';
+#endif
+      natEnc = GSPrivateCStringEncoding(encbuf);
     }
   return natEnc;
 }
 
+/* Returns the C-String encoding as used by ICU library functions.
+ * We can use this to convert strings produced by those functions
+ * to NSString objects.
+ */
+NSStringEncoding
+GSPrivateICUCStringEncoding()
+{
+  if (icuEnc == GSUndefinedEncoding)
+    {
+      const char        *encoding = 0;
+#if HAVE_UNICODE_UCNV_H
+      const char *defaultName;
+      UErrorCode err = U_ZERO_ERROR;
+
+      defaultName = ucnv_getDefaultName ();
+      encoding = ucnv_getStandardName (defaultName, "MIME", &err);
+      if (0 == encoding)
+        {
+          encoding = ucnv_getStandardName (defaultName, "IANA", &err);
+        }
+#endif
+      icuEnc = GSPrivateCStringEncoding(encoding);
+    }
+  return icuEnc;
+}
+
