@@ -124,13 +124,6 @@ cacheEqual(id A, id B)
 #define GSI_MAP_VTYPES	GSUNION_NSINT
 #define	GSI_MAP_NOCLEAN	1
 
-#if	GS_WITH_GC
-#include	<gc/gc_typed.h>
-static GC_descr	nodeDesc;	// Type descriptor for map node.
-#define	GSI_MAP_NODES(M, X) \
-(GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
-#endif
-
 #include "GNUstepBase/GSIMap.h"
 
 static NSLock		*attrLock = nil;
@@ -144,6 +137,8 @@ static IMP		unlockImp;
 #define	AUNLOCK() if (attrLock != nil) (*unlockImp)(attrLock, unlockSel)
 
 @class  GSCachedDictionary;
+@interface GSCachedDictionary : NSDictionary    // Help the compiler
+@end
 @protocol       GSCachedDictionary
 - (void) _uncache;
 @end
@@ -463,15 +458,6 @@ _attributesAtIndexEffectiveRange(
       NSMutableArray	*a;
       NSDictionary	*d;
 
-#if	GS_WITH_GC
-      /* We create a typed memory descriptor for map nodes.
-       * Only the pointer to the key needs to be scanned.
-       */
-      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
-      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
-      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
-#endif
-
       infSel = @selector(newWithZone:value:at:);
       addSel = @selector(addObject:);
       cntSel = @selector(count);
