@@ -96,9 +96,9 @@
 #define	NETBUF_SIZE	(1024 * 16)
 #define	READ_SIZE	NETBUF_SIZE*10
 
-static GSFileHandle*	fh_stdin = nil;
-static GSFileHandle*	fh_stdout = nil;
-static GSFileHandle*	fh_stderr = nil;
+static GSFileHandle     *fh_stdin = nil;
+static GSFileHandle     *fh_stdout = nil;
+static GSFileHandle     *fh_stderr = nil;
 
 @interface      GSTcpTune : NSObject
 - (int) delay;
@@ -359,11 +359,14 @@ static GSTcpTune        *tune = nil;
   DESTROY(address);
   DESTROY(service);
   DESTROY(protocol);
-
-  [self finalize];
-
   DESTROY(readInfo);
   DESTROY(writeInfo);
+
+  /* Finalize *after* destroying readInfo and writeInfo so that, if the
+   * file handle needs to be closed, we don't generate any notifications
+   * containing the deallocated object.  Tnanks to david for this fix.
+   */
+  [self finalize];
   [super dealloc];
 }
 
@@ -1205,7 +1208,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
       if (fstat(desc, &sbuf) < 0)
 	{
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 	  /* On windows, an fstat will fail if the descriptor is a pipe
 	   * or socket, so we simply mark the descriptor as not being a
 	   * standard file.
