@@ -547,15 +547,20 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	}
     }
 
+  buf = [[s dataUsingEncoding: NSISOLatin1StringEncoding] mutableCopy];
+
   enumerator = NSEnumerateMapTable(wProperties);
   while (NSNextMapEnumeratorPair(&enumerator, (void **)(&key), (void**)&val))
     {
-      [s appendFormat: @"%@: %@\r\n", key, val];
+      GSMimeHeader      *h;
+
+      h = [[GSMimeHeader alloc] initWithName: key value: val parameters: nil];
+      [buf appendData: [h rawMimeDataPreservingCase: YES foldedAt: 0]];
+      RELEASE(h);
     }
   NSEndMapTableEnumeration(&enumerator);
 
-  [s appendString: @"\r\n"];
-  buf = [[s dataUsingEncoding: NSASCIIStringEncoding] mutableCopy];
+  [buf appendBytes: "\r\n" length: 2];
 
   /*
    * Append any data to be sent
@@ -1642,6 +1647,9 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
  *     to 8080 for <code>http</code> and 4430 for <code>https</code>.
  *   </item>
  *   <item>
+ *     Any GSTLS... key to control TLS behavior
+ *   </item>
+ *   <item>
  *     Any NSHTTPProperty... key
  *   </item>
  * </list>
@@ -1655,6 +1663,7 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
         format: @"%@ %p with invalid key", NSStringFromSelector(_cmd), self];
     }
   if ([propertyKey hasPrefix: @"GSHTTPProperty"]
+    || [propertyKey hasPrefix: @"GSTLS"]
     || [propertyKey hasPrefix: @"NSHTTPProperty"])
     {
       if (property == nil)
