@@ -61,6 +61,9 @@ typeSize(const char* type)
       case _C_ULNG_LNG:	return sizeof(unsigned long long);
       case _C_FLT:	return sizeof(float);
       case _C_DBL:	return sizeof(double);
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:	return sizeof(_Bool);
+#endif
       case _C_PTR:	return sizeof(void*);
       case _C_CHARPTR:	return sizeof(char*);
       case _C_BFLD:
