@@ -27,7 +27,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2016-05-19 08:21:36 +0100 (Thu, 19 May 2016) $ $Revision: 39779 $
+   $Date$ $Revision$
    */
 
 #import "common.h"
@@ -1204,6 +1204,10 @@ compare(id elem1, id elem2, void* context)
               index--;
             }
         }
+      if (index >= NSMaxRange(range))
+        {
+          return NSNotFound;
+        }
       /*
        * For a search from the left, we'd have the correct index anyways. Check
        * whether it's equal to the key and return NSNotFound otherwise
