@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileHandle class reference</title>
-   $Date: 2016-06-28 11:40:33 +0100 (Tue, 28 Jun 2016) $ $Revision: 39937 $
+   $Date$ $Revision$
    */
 
 #import "common.h"
@@ -848,6 +848,16 @@ NSString * const NSFileHandleOperationException
   return YES;
 }
 
+- (NSString*) sslIssuer
+{
+  return nil;
+}
+
+- (NSString*) sslOwner
+{
+  return nil;
+}
+
 - (void) sslSetCertificate: (NSString*)certFile
                 privateKey: (NSString*)privateKey
                  PEMpasswd: (NSString*)PEMpasswd
@@ -1045,6 +1055,16 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
     }
 }
 
+- (NSString*) sslIssuer
+{
+  return [session issuer];
+}
+
+- (NSString*) sslOwner
+{
+  return [session owner];
+}
+
 - (NSString*) sslSetOptions: (NSDictionary*)options
 {
   if (isStandardFile == YES)
