@@ -59,6 +59,7 @@ NSString * const NSHTTPCookiePort = @"Port";
 NSString * const NSHTTPCookieSecure = @"Secure";
 NSString * const NSHTTPCookieValue = @"Value";
 NSString * const NSHTTPCookieVersion = @"Version";
+static NSString * const HTTPCookieHTTPOnly = @"HTTPOnly";
 
 // Internal data storage
 typedef struct {
@@ -328,6 +329,11 @@ static NSRange GSRangeOfCookie(NSString *string);
   return [[this->_properties objectForKey: NSHTTPCookieSecure] boolValue];
 }
 
+- (BOOL) isHTTPOnly
+{
+  return [[this->_properties objectForKey: HTTPCookieHTTPOnly] boolValue];
+}
+
 - (BOOL) isSessionOnly
 {
   return [[this->_properties objectForKey: NSHTTPCookieDiscard] boolValue];
@@ -690,6 +696,9 @@ _setCookieKey(NSMutableDictionary *dict, NSString *key, NSString *value)
   else if ([[key lowercaseString] isEqual: @"secure"])
     [dict setObject: [NSNumber numberWithBool: YES] 
 	     forKey: NSHTTPCookieSecure];
+  else if ([[key lowercaseString] isEqual:@"httponly"])
+    [dict setObject: [NSNumber numberWithBool: YES]
+             forKey: HTTPCookieHTTPOnly];
   else if ([[key lowercaseString] isEqual: @"version"])
     [dict setObject: value forKey: NSHTTPCookieVersion];
   return YES;
@@ -736,7 +745,19 @@ GSPropertyListFromCookieFormat(NSString *string, int version)
 	}
       else
 	{
+	  unsigned int oldpos = pld->pos;
+	  unsigned int keyvalpos = 0;
+	  id keyval = parseUnquotedString(pld, ';');
+	  keyvalpos = pld->pos;
+	  pld->pos = oldpos;
 	  key = parseUnquotedString(pld, '=');
+
+	  // Detect value-less cookies like HTTPOnly; and Secure;
+	  if ([keyval length] < [key length])
+	    {
+	      pld->pos = keyvalpos;
+	      key = keyval;
+	    }
 	}
       if (key == nil)
 	{
