@@ -731,14 +731,8 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
 	}
       else
 	{
-#if	GS_WITH_GC
-	  _data = (GSIArray)NSAllocateCollectable(sizeof(GSIArray_t),
-	    NSScannedOption);
-	  GSIArrayInitWithZoneAndCapacity(_array, 0, 1);
-#else
 	  _data = (GSIArray)NSZoneMalloc([self zone], sizeof(GSIArray_t));
 	  GSIArrayInitWithZoneAndCapacity(_array, [self zone], 1);
-#endif
 	  GSIArrayAddItem(_array, (GSIArrayItem)aRange);
 	}
     }
@@ -759,14 +753,8 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
 	{
 	  NSUInteger i;
 
-#if	GS_WITH_GC
-	  _data = (GSIArray)NSAllocateCollectable(sizeof(GSIArray_t),
-	    NSScannedOption);
-	  GSIArrayInitWithZoneAndCapacity(_array, 0, 1);
-#else
 	  _data = (GSIArray)NSZoneMalloc([self zone], sizeof(GSIArray_t));
 	  GSIArrayInitWithZoneAndCapacity(_array, [self zone], count);
-#endif
 	  for (i = 0; i < count; i++)
 	    {
 	      GSIArrayAddItem(_array, GSIArrayItemAtIndex(_other, i));
@@ -1043,14 +1031,8 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
     }
   if (_array == 0)
     {
-#if	GS_WITH_GC
-      _data = (GSIArray)NSAllocateCollectable(sizeof(GSIArray_t),
-	NSScannedOption);
-      GSIArrayInitWithZoneAndCapacity(_array, 0, 1);
-#else
       _data = (GSIArray)NSZoneMalloc([self zone], sizeof(GSIArray_t));
       GSIArrayInitWithZoneAndCapacity(_array, [self zone], 1);
-#endif
     }
 
   pos = posForIndex(_array, aRange.location);
