@@ -56,6 +56,44 @@
   [_name release];\
   [super dealloc];\
 }
+
+#if     defined(HAVE_PTHREAD_MUTEX_OWNER)
+
+#define	MDESCRIPTION \
+- (NSString*) description\
+{\
+  if (_mutex.__data.__owner)\
+    {\
+      if (_name == nil)\
+        {\
+          return [NSString stringWithFormat: @"%@ (locked by %llu)",\
+            [super description], (unsigned long long)_mutex.__data.__owner];\
+        }\
+      return [NSString stringWithFormat: @"%@ '%@' (locked by %llu)",\
+        [super description], _name, (unsigned long long)_mutex.__data.__owner];\
+    }\
+  else\
+    {\
+      if (_name == nil)\
+        {\
+          return [super description];\
+        }\
+      return [NSString stringWithFormat: @"%@ '%@'",\
+        [super description], _name];\
+    }\
+}
+
+#define	MISLOCKED \
+- (BOOL) isLockedByCurrentThread\
+{\
+  if (GSPrivateThreadID() == (NSUInteger)_mutex.__data.__owner)\
+    return YES;\
+  else\
+    return NO; \
+}
+
+#else
+
 #define	MDESCRIPTION \
 - (NSString*) description\
 {\
@@ -66,11 +104,22 @@
   return [NSString stringWithFormat: @"%@ '%@'",\
     [super description], _name];\
 }
+
+#define	MISLOCKED \
+- (BOOL) isLockedByCurrentThread\
+{\
+  [NSException raise: NSGenericException format: @"Not supported"];\
+  return NO;\
+}
+
+#endif
+
 #define MFINALIZE \
 - (void) finalize\
 {\
   pthread_mutex_destroy(&_mutex);\
 }
+
 #define MNAME \
 - (void) setName: (NSString*)newName\
 {\
@@ -80,6 +129,7 @@
 {\
   return _name;\
 }
+
 #define	MLOCK \
 - (void) lock\
 {\
@@ -94,6 +144,7 @@
       _NSLockError(self, _cmd, YES);\
     }\
 }
+
 #define	MLOCKBEFOREDATE \
 - (BOOL) lockBeforeDate: (NSDate*)limit\
 {\
@@ -108,12 +159,14 @@
     } while ([limit timeIntervalSinceNow] > 0);\
   return NO;\
 }
+
 #define	MTRYLOCK \
 - (BOOL) tryLock\
 {\
   int err = pthread_mutex_trylock(&_mutex);\
   return (0 == err) ? YES : NO;\
 }
+
 #define	MUNLOCK \
 - (void) unlock\
 {\
@@ -202,6 +255,7 @@ MFINALIZE
   return self;
 }
 
+MISLOCKED
 MLOCK
 
 - (BOOL) lockBeforeDate: (NSDate*)limit
@@ -250,6 +304,7 @@ MFINALIZE
   return self;
 }
 
+MISLOCKED
 MLOCK
 MLOCKBEFOREDATE
 MNAME
@@ -295,6 +350,7 @@ MDESCRIPTION
   return self;
 }
 
+MISLOCKED
 MLOCK
 MLOCKBEFOREDATE
 MNAME
@@ -381,6 +437,11 @@ MUNLOCK
   return self;
 }
 
+- (BOOL) isLockedByCurrentThread
+{
+  return [_condition isLockedByCurrentThread];
+}
+
 - (void) lock
 {
   [_condition lock];
