@@ -27,19 +27,6 @@
 
 #import "common.h"
 #import "Foundation/NSArray.h"
-#import "GNUstepBase/NSObject+GNUstepBase.h"
-
-@interface NSObject (NSComparisonMethods)
-- (BOOL) doesContain: (id) object;
-- (BOOL) isCaseInsensitiveLike: (id) object;
-- (BOOL) isEqualTo: (id) object;
-- (BOOL) isGreaterThan: (id) object;
-- (BOOL) isGreaterThanOrEqualTo: (id) object;
-- (BOOL) isLessThan: (id) object;
-- (BOOL) isLessThanOrEqualTo: (id) object;
-- (BOOL) isLike: (NSString *)object;
-- (BOOL) isNotEqualTo: (id) object;
-@end
 
 @implementation NSObject (NSComparisonMethods)
 - (BOOL) doesContain: (id) object
