@@ -1,5 +1,5 @@
 /** Implementation of NSObject for GNUStep
-   Copyright (C) 1994-2010 Free Software Foundation, Inc.
+   Copyright (C) 1994-2017 Free Software Foundation, Inc.
 
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
    Date: August 1994
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2016-11-18 06:49:16 +0000 (Fri, 18 Nov 2016) $ $Revision: 40225 $
+   $Date$ $Revision$
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -77,6 +77,17 @@
 #endif
 #endif
 
+
+/* platforms which do not support weak */
+#if (__GNUC__ == 3) && defined (__WIN32)
+#define WEAK_ATTRIBUTE
+#undef SUPPORT_WEAK
+#else
+/* all platforms which support weak */
+#define WEAK_ATTRIBUTE __attribute__((weak))
+#define SUPPORT_WEAK 1
+#endif
+
 /* When this is `YES', every call to release/autorelease, checks to
    make sure isn't being set up to release itself too many times.
    This does not need mutex protection. */
@@ -125,6 +136,7 @@ BOOL	NSDeallocateZombies = NO;
 static Class		zombieClass = Nil;
 static NSMapTable	*zombieMap = 0;
 
+#ifndef OBJC_CAP_ARC
 static void GSMakeZombie(NSObject *o, Class c)
 {
   object_setClass(o, zombieClass);
@@ -135,6 +147,7 @@ static void GSMakeZombie(NSObject *o, Class c)
       [allocationLock unlock];
     }
 }
+#endif
 
 static void GSLogZombie(id o, SEL sel)
 {
@@ -444,15 +457,25 @@ struct obj_layout {
 };
 typedef	struct obj_layout *obj;
 
-/**
- * Examines the extra reference count for the object and, if non-zero
- * decrements it, otherwise leaves it unchanged.<br />
- * Returns a flag to say whether the count was zero
- * (and hence whether the extra reference count was decremented).<br />
- * This function is used by the [NSObject-release] method.
+/*
+ * These symbols are provided by newer versions of the GNUstep Objective-C
+ * runtime.  When linked against an older version, we will use our internal
+ * versions.
  */
-inline BOOL
-NSDecrementExtraRefCountWasZero(id anObject)
+WEAK_ATTRIBUTE
+BOOL objc_release_fast_no_destroy_np(id anObject);
+
+WEAK_ATTRIBUTE
+void objc_release_fast_np(id anObject);
+
+WEAK_ATTRIBUTE
+size_t object_getRetainCount_np(id anObject);
+
+WEAK_ATTRIBUTE
+id objc_retain_fast_np(id anObject);
+
+
+static BOOL objc_release_fast_no_destroy_internal(id anObject)
 {
   if (double_release_check_enabled)
     {
@@ -483,6 +506,9 @@ NSDecrementExtraRefCountWasZero(id anObject)
          * have been greater than zero)
          */
         (((obj)anObject)[-1].retained) = 0;
+#  ifdef OBJC_CAP_ARC
+        objc_delete_weak_refs(anObject);
+#  endif
         return YES;
       }
 #else	/* GSATOMICREAD */
@@ -491,6 +517,9 @@ NSDecrementExtraRefCountWasZero(id anObject)
     [theLock lock];
     if (((obj)anObject)[-1].retained == 0)
       {
+#  ifdef OBJC_CAP_ARC
+        objc_delete_weak_refs(anObject);
+#  endif
         [theLock unlock];
         return YES;
       }
@@ -505,6 +534,73 @@ NSDecrementExtraRefCountWasZero(id anObject)
   return NO;
 }
 
+static BOOL release_fast_no_destroy(id anObject)
+{
+#ifdef SUPPORT_WEAK
+  if (objc_release_fast_no_destroy_np)
+    {
+      return objc_release_fast_no_destroy_np(anObject);
+    }
+  else
+#endif
+    {
+      return objc_release_fast_no_destroy_internal(anObject);
+    }
+}
+
+static void objc_release_fast_np_internal(id anObject)
+{
+  if (release_fast_no_destroy(anObject))
+    {
+      [anObject dealloc];
+    }
+}
+
+static void release_fast(id anObject)
+{
+#ifdef SUPPORT_WEAK
+  if (objc_release_fast_np)
+    {
+      objc_release_fast_np(anObject);
+    }
+  else
+#endif
+    {
+      objc_release_fast_np_internal(anObject);
+    }
+}
+
+/**
+ * Examines the extra reference count for the object and, if non-zero
+ * decrements it, otherwise leaves it unchanged.<br />
+ * Returns a flag to say whether the count was zero
+ * (and hence whether the extra reference count was decremented).<br />
+ */
+inline BOOL
+NSDecrementExtraRefCountWasZero(id anObject)
+{
+  return release_fast_no_destroy(anObject);
+}
+
+size_t object_getRetainCount_np_internal(id anObject)
+{
+  return ((obj)anObject)[-1].retained + 1;
+}
+
+size_t getRetainCount(id anObject)
+{
+#ifdef SUPPORT_WEAK
+  if (object_getRetainCount_np)
+    {
+      return object_getRetainCount_np(anObject);
+    }
+  else
+#endif
+    {
+      return object_getRetainCount_np_internal(anObject);
+    }
+}
+
 /**
  * Return the extra reference count of anObject (a value in the range
  * from 0 to the maximum unsigned integer value minus one).<br />
@@ -513,7 +609,93 @@ NSDecrementExtraRefCountWasZero(id anObject)
 inline NSUInteger
 NSExtraRefCount(id anObject)
 {
-  return ((obj)anObject)[-1].retained;
+  return getRetainCount(anObject) - 1;
+}
+
+/**
+ * Increments the extra reference count for anObject.<br />
+ * The GNUstep version raises an exception if the reference count
+ * would be incremented to too large a value.<br />
+ * This is used by the [NSObject-retain] method.
+ */
+static id objc_retain_fast_np_internal(id anObject)
+{
+  BOOL  tooFar = NO;
+
+#if	defined(GSATOMICREAD)
+  /* I've seen comments saying that some platforms only support up to
+   * 24 bits in atomic locking, so raise an exception if we try to
+   * go beyond 0xfffffe.
+   */
+  if (GSAtomicIncrement((gsatomic_t)&(((obj)anObject)[-1].retained))
+    > 0xfffffe)
+    {
+      tooFar = YES;
+    }
+#else	/* GSATOMICREAD */
+  NSLock *theLock = GSAllocationLockForObject(anObject);
+
+  [theLock lock];
+  if (((obj)anObject)[-1].retained > 0xfffffe)
+    {
+      tooFar = YES;
+    }
+  else
+    {
+      ((obj)anObject)[-1].retained++;
+    }
+  [theLock unlock];
+#endif	/* GSATOMICREAD */
+  if (YES == tooFar)
+    {
+      static NSHashTable        *overrun = nil;
+
+      /* We store this instance in a hash table so that we will only raise
+       * an exception for it once (and can therefore expect to log the instance
+       * as part of the exception derscription without recursion).
+       * NB. The hash table does not retain the object, so the code in the
+       * lock protected region below should be safe anyway.
+       */
+      [gnustep_global_lock lock];
+      if (nil == overrun)
+        {
+          overrun = NSCreateHashTable(NSNonRetainedObjectHashCallBacks, 0);
+        }
+      if (0 == NSHashGet(overrun, anObject))
+        {
+          NSHashInsert(overrun, anObject);
+        }
+      else
+        {
+          tooFar = NO;
+        }
+      [gnustep_global_lock lock];
+      if (YES == tooFar)
+        {
+          NSString      *base;
+
+          base = [NSString stringWithFormat: @"<%s: %p>",
+            class_getName([anObject class]), anObject];
+          [NSException raise: NSInternalInconsistencyException
+            format: @"NSIncrementExtraRefCount() asked to increment too far"
+            @" for %@ - %@", base, anObject];
+        }
+    }
+  return anObject;
+}
+
+static id retain_fast(id anObject)
+{
+#ifdef SUPPORT_WEAK
+  if (objc_retain_fast_np)
+    {
+      return objc_retain_fast_np(anObject);
+    }
+  else
+#endif
+    {
+      return objc_retain_fast_np_internal(anObject);
+    }
 }
 
 /**
@@ -525,30 +707,7 @@ NSExtraRefCount(id anObject)
 inline void
 NSIncrementExtraRefCount(id anObject)
 {
-#if	defined(GSATOMICREAD)
-  /* I've seen comments saying that some platforms only support up to
-   * 24 bits in atomic locking, so raise an exception if we try to
-   * go beyond 0xfffffe.
-   */
-  if (GSAtomicIncrement((gsatomic_t)&(((obj)anObject)[-1].retained))
-    > 0xfffffe)
-    {
-      [NSException raise: NSInternalInconsistencyException
-        format: @"NSIncrementExtraRefCount() asked to increment too far"];
-    }
-#else	/* GSATOMICREAD */
-  NSLock *theLock = GSAllocationLockForObject(anObject);
-
-  [theLock lock];
-  if (((obj)anObject)[-1].retained > 0xfffffe)
-    {
-      [theLock unlock];
-      [NSException raise: NSInternalInconsistencyException
-        format: @"NSIncrementExtraRefCount() asked to increment too far"];
-    }
-  ((obj)anObject)[-1].retained++;
-  [theLock unlock];
-#endif	/* GSATOMICREAD */
+   retain_fast(anObject);
 }
 
 #ifndef	NDEBUG
@@ -559,6 +718,8 @@ NSIncrementExtraRefCount(id anObject)
 #define	AREM(c, o)
 #endif
 
+
+#ifndef OBJC_CAP_ARC
 static SEL cxx_construct, cxx_destruct;
 
 /**
@@ -592,6 +753,7 @@ callCXXConstructors(Class aClass, id anObject)
     }
   return constructor;
 }
+#endif
 
 
 /*
@@ -605,6 +767,9 @@ callCXXConstructors(Class aClass, id anObject)
 inline id
 NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
 {
+#ifdef OBJC_CAP_ARC
+  return class_createInstance(aClass, extraBytes);
+#else
   id	new;
   int	size;
 
@@ -637,17 +802,21 @@ NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
   callCXXConstructors(aClass, new);
 
   return new;
+#endif
 }
 
 inline void
 NSDeallocateObject(id anObject)
 {
+
   Class aClass = object_getClass(anObject);
 
   if ((anObject != nil) && !class_isMetaClass(aClass))
     {
+#ifndef OBJC_CAP_ARC
       obj	o = &((obj)anObject)[-1];
       NSZone	*z = NSZoneFromPointer(o);
+#endif
 
       /* Call the default finalizer to handle C++ destructors.
        */
@@ -656,16 +825,37 @@ NSDeallocateObject(id anObject)
       AREM(aClass, (id)anObject);
       if (NSZombieEnabled == YES)
 	{
+#ifdef OBJC_CAP_ARC
+	  if (0 != zombieMap)
+	    {
+	      [allocationLock lock];
+	      NSMapInsert(zombieMap, (void*)anObject, (void*)aClass);
+	      [allocationLock unlock];
+	    }
+	  if (NSDeallocateZombies == YES)
+	    {
+	      object_dispose(anObject);
+	    }
+	  else
+	    {
+	      object_setClass(anObject, zombieClass);
+	    }
+#else
 	  GSMakeZombie(anObject, aClass);
 	  if (NSDeallocateZombies == YES)
 	    {
 	      NSZoneFree(z, o);
 	    }
+#endif
 	}
       else
 	{
+#ifdef OBJC_CAP_ARC
+	  object_dispose(anObject);
+#else
 	  object_setClass((id)anObject, (Class)(void*)0xdeadface);
 	  NSZoneFree(z, o);
+#endif
 	}
     }
   return;
@@ -754,7 +944,7 @@ objc_create_block_classes_as_subclasses_of(Class super);
 #ifdef OBJC_CAP_ARC
 static id gs_weak_load(id obj)
 {
-	return [obj retainCount] > 0 ? obj : nil;
+  return [obj retainCount] > 0 ? obj : nil;
 }
 #endif
 
@@ -1160,6 +1350,7 @@ static id gs_weak_load(id obj)
 
 - (void) finalize
 {
+#ifndef OBJC_CAP_ARC
   Class	destructorClass = Nil;
   IMP	  destructor = 0;
   /*
@@ -1214,6 +1405,7 @@ static id gs_weak_load(id obj)
 	}
     }
   return;
+#endif
 }
 
 /**
@@ -1856,13 +2048,7 @@ static id gs_weak_load(id obj)
  */
 - (oneway void) release
 {
-  if (NSDecrementExtraRefCountWasZero(self))
-    {
-#  ifdef OBJC_CAP_ARC
-      objc_delete_weak_refs(self);
-#  endif
-      [self dealloc];
-    }
+  release_fast(self);
 }
 
 /**
@@ -1921,8 +2107,7 @@ static id gs_weak_load(id obj)
  */
 - (id) retain
 {
-  NSIncrementExtraRefCount(self);
-  return self;
+  return retain_fast(self);
 }
 
 /**
@@ -1946,7 +2131,7 @@ static id gs_weak_load(id obj)
  */
 - (NSUInteger) retainCount
 {
-  return NSExtraRefCount(self) + 1;
+  return getRetainCount(self);
 }
 
 /**
