@@ -362,10 +362,10 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 - (BOOL) evaluateWithObject: (id)object
-	  substitutionVariables: (GS_GENERIC_CLASS(NSDictionary, NSString*, id)*)variables
+      substitutionVariables: (GS_GENERIC_CLASS(NSDictionary, NSString*, id)*)variables
 {
   return [[self predicateWithSubstitutionVariables: variables]
-		   evaluateWithObject: object];
+                                evaluateWithObject: object];
 }
 #endif
 - (Class) classForCoder
@@ -898,15 +898,6 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   BOOL leftIsNil;
   BOOL rightIsNil;
 
-  if (leftResult == evaluatedObjectExpression)
-    {
-      leftResult = object;
-    }
-  if (rightResult == evaluatedObjectExpression)
-    {
-      rightResult = object;
-    }
-
   leftIsNil = (leftResult == nil || [leftResult isEqual: [NSNull null]]);
   rightIsNil = (rightResult == nil || [rightResult isEqual: [NSNull null]]);
   if (leftIsNil || rightIsNil)
@@ -949,17 +940,29 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   switch (_type)
     {
       case NSLessThanPredicateOperatorType:
-	return ([leftResult compare: rightResult] == NSOrderedAscending)
-          ? YES : NO;
+        {
+          double ld = [leftResult doubleValue];
+          double rd = [rightResult doubleValue];
+          return (ld < rd) ? YES : NO;
+        }
       case NSLessThanOrEqualToPredicateOperatorType:
-	return ([leftResult compare: rightResult] != NSOrderedDescending)
-          ? YES : NO;
+        {
+          double ld = [leftResult doubleValue];
+          double rd = [rightResult doubleValue];
+          return (ld <= rd) ? YES : NO;
+        }
       case NSGreaterThanPredicateOperatorType:
-	return ([leftResult compare: rightResult] == NSOrderedDescending)
-          ? YES : NO;
+        {
+          double ld = [leftResult doubleValue];
+          double rd = [rightResult doubleValue];
+          return (ld > rd) ? YES : NO;
+        }
       case NSGreaterThanOrEqualToPredicateOperatorType:
-	return ([leftResult compare: rightResult] != NSOrderedAscending)
-          ? YES : NO;
+        {
+          double ld = [leftResult doubleValue];
+          double rd = [rightResult doubleValue];
+          return (ld >= rd) ? YES : NO;
+        }
       case NSEqualToPredicateOperatorType:
 	return [leftResult isEqual: rightResult];
       case NSNotEqualToPredicateOperatorType:
@@ -1070,11 +1073,6 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
       NSEnumerator *e;
       id value;
 
-      if (leftValue == evaluatedObjectExpression)
-	{
-	  leftValue = object;
-	}
-
       if (![leftValue respondsToSelector: @selector(objectEnumerator)])
         {
           [NSException raise: NSInvalidArgumentException 
@@ -1356,7 +1354,7 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
 - (id) expressionValueWithObject: (id)object
 			 context: (NSMutableDictionary *)context
 {
-  return self;
+  return object;
 }
 
 - (id) _expressionWithSubstitutionVariables: (NSDictionary *)variables
@@ -1364,6 +1362,10 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   return self;
 }
 
+- (NSString *) keyPath
+{
+  return @"SELF";
+}
 @end
 
 @implementation GSVariableExpression
@@ -1523,7 +1525,7 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
     }
   // apply method selector
   return [self performSelector: _selector
-                      withObject: eargs];
+                    withObject: eargs];
 }
 
 - (void) dealloc;
@@ -1543,6 +1545,20 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   return copy;
 }
 
+- (NSEnumerator*) _enum: (NSArray *)expressions
+{
+  id    o;
+
+  /* Check to see if this is aggregating over a collection.
+   */
+  if (1 == _argc && [(o = [expressions lastObject])
+    respondsToSelector: @selector(objectEnumerator)])
+    {
+      return [o objectEnumerator];
+    }
+  return [expressions objectEnumerator];
+}
+
 - (id) _expressionWithSubstitutionVariables: (NSDictionary *)variables
 {
   NSMutableArray *args = [NSMutableArray arrayWithCapacity: _argc];
@@ -1593,7 +1609,8 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   id left = [expressions objectAtIndex: 0];
   id right = [expressions objectAtIndex: 1];
 
-  return [NSNumber numberWithDouble: pow([left doubleValue], [right doubleValue])];
+  return [NSNumber numberWithDouble:
+    pow([left doubleValue], [right doubleValue])];
 }
 
 - (id) _eval__mul: (NSArray *)expressions
@@ -1637,42 +1654,47 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
 
 - (id) _eval_avg: (NSArray *)expressions 
 {
-  unsigned int i;
-  double sum = 0.0;
+  NSEnumerator  *e = [self _enum: expressions];
+  double        sum = 0.0;
+  unsigned      count = 0;
+  id            o;
     
-  for (i = 0; i < _argc; i++)
+  while (nil != (o = [e nextObject]))
     {
-      sum += [[expressions objectAtIndex: i] doubleValue];
+      sum += [o doubleValue];
+      count++;
     }
-  return [NSNumber numberWithDouble: sum / _argc];
+  return [NSNumber numberWithDouble: sum / count];
 }
 
-- (id) _eval_sum: (NSArray *)expressions
+- (id) _eval_max: (NSArray *)expressions
 {
-  unsigned int i;
-  double sum = 0.0;
-    
-  for (i = 0; i < _argc; i++)
+  NSEnumerator  *e = [self _enum: expressions];
+  id            o = [e nextObject];
+  double        max = (nil == o) ? 0.0 : [o doubleValue];
+  double        cur;
+  
+  while (nil != (o = [e nextObject]))
     {
-      sum += [[expressions objectAtIndex: i] doubleValue];
+      cur = [o doubleValue];
+      if (max < cur)
+        {
+          max = cur;
+        }
     }
-  return [NSNumber numberWithDouble: sum];
+  return [NSNumber numberWithDouble: max];
 }
 
 - (id) _eval_min: (NSArray *)expressions
 {
-  unsigned int i;
-  double min = 0.0;
-  double cur;
-  
-  if (_argc > 0)
-    {
-      min = [[expressions objectAtIndex: 0] doubleValue];
-    }
+  NSEnumerator  *e = [self _enum: expressions];
+  id            o = [e nextObject];
+  double        min = (nil == o ? 0.0 : [o doubleValue]);
+  double        cur;
 
-  for (i = 1; i < _argc; i++)
+  while (nil != (o = [e nextObject]))
     {
-      cur = [[expressions objectAtIndex: i] doubleValue];
+      cur = [o doubleValue];
       if (min > cur)
         {
           min = cur;
@@ -1681,26 +1703,17 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
   return [NSNumber numberWithDouble: min];
 }
 
-- (id) _eval_max: (NSArray *)expressions
+- (id) _eval_sum: (NSArray *)expressions
 {
-  unsigned int i;
-  double max = 0.0;
-  double cur;
-  
-  if (_argc > 0)
-    {
-      max = [[expressions objectAtIndex: 0] doubleValue];
-    }
+  NSEnumerator  *e = [self _enum: expressions];
+  double        sum = 0.0;
+  id            o;
 
-  for (i = 1; i < _argc; i++)
+  while (nil != (o = [e nextObject]))
     {
-      cur = [[expressions objectAtIndex: i] doubleValue];
-      if (max < cur)
-        {
-          max = cur;
-        }
+      sum += [o doubleValue];
     }
-  return [NSNumber numberWithDouble: max];
+  return [NSNumber numberWithDouble: sum];
 }
 
 // add arithmetic functions: average, median, mode, stddev, sqrt, log, ln, exp, floor, ceiling, abs, trunc, random, randomn, now
@@ -2514,10 +2527,17 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
                           format: @"Invalid right keypath: %@", left];
             }
 
-          // concatenate
-          left = [NSExpression expressionForKeyPath:
-                    [NSString stringWithFormat: @"%@.%@",
-                              [left keyPath], [right keyPath]]];
+          if (evaluatedObjectExpression != left)
+            {
+              // concatenate
+              left = [NSExpression expressionForKeyPath:
+                        [NSString stringWithFormat: @"%@.%@",
+                                  [left keyPath], [right keyPath]]];
+            }
+          else
+            {
+              left = [NSExpression expressionForKeyPath: [right keyPath]];
+            }
         }
       else
         {
