@@ -1,5 +1,5 @@
 /** Implementation for NSProcessInfo for GNUStep
-   Copyright (C) 1995-2014 Free Software Foundation, Inc.
+   Copyright (C) 1995-2017 Free Software Foundation, Inc.
 
    Written by:  Georg Tuparev <Tuparev@EMBL-Heidelberg.de>
                 Heidelberg, Germany
@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProcessInfo class reference</title>
-   $Date: 2016-04-17 16:18:43 +0100 (Sun, 17 Apr 2016) $ $Revision: 39674 $
+   $Date$ $Revision$
 */
 
 /*************************************************************************
@@ -1465,6 +1465,89 @@ static void determineOperatingSystem()
   return availMem;
 }
 
+- (NSUInteger) systemUptime
+{
+  NSUInteger uptime = 0;
+
+#if	defined(_WIN32)
+
+#if _WIN32_WINNT < 0x0600 /* less than Vista */
+  uptime = GetTickCount() / 1000;
+#else
+  uptime = GetTickCount64() / 1000;
+#endif
+  
+#elif	defined(HAVE_SYSCTLBYNAME)
+  struct timeval	tval;
+  size_t		len = sizeof(tval);
+
+  if (sysctlbyname("kern.boottime", &tval, &len, 0, 0) == 0)
+    {
+      uptime = tval.tv_sec;
+    }
+#elif	defined(HAVE_PROCFS)
+  NSFileManager *fileManager = [NSFileManager defaultManager];
+
+  if ([fileManager fileExistsAtPath: @"/proc/uptime"])
+    {
+      NSString *uptimeContent = [NSString
+	stringWithContentsOfFile: @"/proc/uptime"];
+      NSString *uptimeString = [[uptimeContent
+	componentsSeparatedByString:@" "] objectAtIndex:0];
+      uptime = [uptimeString intValue];
+    }
+#else
+#warning	"no known way to determine uptime on this system"
+#endif
+
+  if (uptime == 0)
+    {
+      NSLog(@"Cannot determine uptime.");
+    }
+
+  return uptime;
+}
+
+- (void) enableSuddenTermination
+{
+  // FIXME: unimplemented
+  return;
+}
+
+- (void) disableSuddenTermination
+{
+  // FIXME: unimplemented
+  return;
+}
+
+- (id) beginActivityWithOptions: (NSActivityOptions)options
+                         reason: (NSString *)reason
+{
+  // FIXME: unimplemented
+  return nil;
+}
+
+- (void) endActivity:(id<NSObject>)activity
+{
+  // FIXME: unimplemented
+  return;
+}
+
+- (void) performActivityWithOptions:(NSActivityOptions)options
+                            reason: (NSString *)reason
+                        usingBlock: (GSPerformActivityBlock)block
+{
+  // FIXME: unimplemented
+  return;
+}
+
+- (void) performExpiringActivityWithReason: (NSString *)reason
+		usingBlock: (GSPerformExpiringActivityBlock)block
+{
+  // FIXME: unimplemented
+  return;
+}
+
 @end
 
 void
