@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2016-03-25 11:15:28 +0000 (Fri, 25 Mar 2016) $ $Revision: 39608 $
+   $Date$ $Revision$
 */
 
 /*
@@ -616,6 +616,12 @@ static NSUInteger	urlAlign;
   return AUTORELEASE([[NSURL alloc] initFileURLWithPath: aPath]);
 }
 
++ (id) fileURLWithPath: (NSString*)aPath isDirectory: (BOOL)isDir
+{
+  return AUTORELEASE([[NSURL alloc] initFileURLWithPath: aPath
+					    isDirectory: isDir]);
+}
+
 + (NSURL*) fileURLWithPathComponents: (NSArray*)components
 {
   return [self fileURLWithPath: [NSString pathWithComponents: components]];
@@ -643,6 +649,14 @@ static NSUInteger	urlAlign;
 				     relativeToURL: aBaseUrl]);
 }
 
++ (id) URLByResolvingAliasFileAtURL: (NSURL*)url 
+                            options: (NSURLBookmarkResolutionOptions)options 
+                              error: (NSError**)error
+{
+  // TODO: unimplemented
+  return nil;
+}
+
 - (id) initFileURLWithPath: (NSString*)aPath
 {
   NSFileManager	*mgr = [NSFileManager defaultManager];
@@ -1547,6 +1561,14 @@ static NSUInteger	urlAlign;
   return [[self path] lastPathComponent];
 }
 
+- (BOOL) getResourceValue: (id*)value 
+                   forKey: (NSString *)key 
+                    error: (NSError**)error
+{
+  // TODO: unimplemented
+  return NO;
+}
+
 - (void) loadResourceDataNotifyingClient: (id)client
 			      usingCache: (BOOL)shouldUseCache
 {
