@@ -24,7 +24,11 @@
 
 #import "common.h"
 
-#define GSInternal              NSXMLDTDNodeInternal
+#if defined(HAVE_LIBXML)
+
+#define GS_XMLNODETYPE	xmlEntity
+#define GSInternal	NSXMLDTDNodeInternal
+
 #import	"NSXMLPrivate.h"
 #import "GSInternal.h"
 GS_PRIVATE_INTERNAL(NSXMLDTDNode)
@@ -35,9 +39,6 @@ GS_PRIVATE_INTERNAL(NSXMLDTDNode)
 {
   if (GS_EXISTS_INTERNAL)
     {
-      [internal->notationName release];
-      [internal->publicID release];
-      [internal->systemID release];
     }
   [super dealloc];
 }
@@ -47,31 +48,52 @@ GS_PRIVATE_INTERNAL(NSXMLDTDNode)
   return internal->DTDKind;
 }
 
-- (id) initWithKind: (NSXMLNodeKind)kind options: (NSUInteger)theOptions
+- (void) _createInternal
 {
-  if (NSXMLEntityDeclarationKind == kind
-    || NSXMLElementDeclarationKind == kind
-    || NSXMLNotationDeclarationKind == kind)
+  GS_CREATE_INTERNAL(NSXMLDTDNode);
+}
+
+- (id) initWithKind: (NSXMLNodeKind)theKind options: (NSUInteger)theOptions
+{
+  if (NSXMLEntityDeclarationKind == theKind
+    || NSXMLElementDeclarationKind == theKind
+    || NSXMLNotationDeclarationKind == theKind)
     {
-      /* Create holder for internal instance variables so that we'll have
-       * all our ivars available rather than just those of the superclass.
-       */
-      GS_CREATE_INTERNAL(NSXMLDTDNode)
+      return [super initWithKind: theKind options: theOptions];
+    }
+  else
+    {
+      [self release];
+      // This cast is here to keep clang quite that expects an init* method to 
+      // return an object of the same class, which is not true here.
+      return (NSXMLDTDNode*)[[NSXMLNode alloc] initWithKind: theKind
+                                                    options: theOptions];
     }
-  return [super initWithKind: kind options: theOptions];
 }
 
 - (id) initWithXMLString: (NSString*)string
 {
-  [self notImplemented: _cmd];
-  return nil;
+  NSXMLDTDNode *result = nil;
+  NSError *error;
+  NSXMLDocument *tempDoc = 
+    [[NSXMLDocument alloc] initWithXMLString: string
+                                     options: 0
+                                       error: &error];
+  if (tempDoc != nil)
+    {
+      result = (NSXMLDTDNode*)RETAIN([tempDoc childAtIndex: 0]);
+      [result detach]; // detach from document.
+    }
+  [tempDoc release];
+  [self release];
+
+  return result;
 }
 
 - (BOOL) isExternal
 {
-  if (internal->systemID != nil)
+  if ([self systemID])
     {
-// FIXME ... libxml integration?
       return YES;
     }
   return NO;
@@ -79,54 +101,39 @@ GS_PRIVATE_INTERNAL(NSXMLDTDNode)
 
 - (NSString*) notationName
 {
-  if (internal->notationName == nil)
-    {
-      [self notImplemented: _cmd];
-    }
-  return internal->notationName;
+  return StringFromXMLStringPtr(internal->node->name);
 }
 
 - (NSString*) publicID
 {
-  if (internal->publicID == nil)
-    {
-      [self notImplemented: _cmd];
-    }
-  return internal->publicID;
+ return StringFromXMLStringPtr(internal->node->ExternalID);
 }
 
-- (void) setDTDKind: (NSXMLDTDNodeKind)kind
+- (void) setDTDKind: (NSXMLDTDNodeKind)nodeKind
 {
-  internal->DTDKind = kind;
-  // FIXME ... libxml integration?
+  internal->DTDKind = nodeKind;
 }
 
 - (void) setNotationName: (NSString*)notationName
 {
-  ASSIGNCOPY(internal->notationName, notationName);
-  // FIXME ... libxml integration?
+  internal->node->name = XMLSTRING(notationName);
 }
 
 - (void) setPublicID: (NSString*)publicID
 {
-  ASSIGNCOPY(internal->publicID, publicID);
-  // FIXME ... libxml integration?
+  internal->node->ExternalID = XMLSTRING(publicID);
 }
 
 - (void) setSystemID: (NSString*)systemID
 {
-  ASSIGNCOPY(internal->systemID, systemID);
-  // FIXME ... libxml integration?
+  internal->node->ExternalID = XMLSTRING(systemID);
 }
 
 - (NSString*) systemID
 {
-  if (internal->systemID == nil)
-    {
-      [self notImplemented: _cmd];
-    }
-  return internal->systemID;
+  return StringFromXMLStringPtr(internal->node->SystemID);
 }
 
 @end
 
+#endif	/* HAVE_LIBXML */
