@@ -245,7 +245,29 @@ isEqualNode(xmlNodePtr nodeA, xmlNodePtr nodeB)
       xmlFree(contentA);
       xmlFree(contentB);
     }
-  // FIXME: Handle more node types
+  if (1) 
+    {
+      xmlChar *contentA = NULL;
+      xmlChar *contentB = NULL;
+
+      // FIXME: Handle more node types
+      if (!isEqualAttributes(nodeA, nodeB))
+	{
+	  return NO;
+	}
+
+      // Get the value of any text node underneath the current element.
+      contentA = xmlNodeGetContent((const xmlNodePtr)nodeA);
+      contentB = xmlNodeGetContent((const xmlNodePtr)nodeB);
+      if (xmlStrcmp(contentA, contentB) != 0)
+	{
+          xmlFree(contentA);
+          xmlFree(contentB);
+	  return NO;
+	}
+      xmlFree(contentA);
+      xmlFree(contentB);
+    }
   
   return YES;
 }
@@ -459,10 +481,8 @@ isEqualTree(xmlNodePtr nodeA, xmlNodePtr nodeB)
 - (void) _removeSubNode: (NSXMLNode *)subNode
 {
   // retain temporarily so we can safely remove from our subNodes list first
-  [subNode retain]; 
+  AUTORELEASE(RETAIN(subNode));
   [internal->subNodes removeObjectIdenticalTo: subNode];
-  // release temporary hold. Apple seems to do an autorelease here.
-  [subNode autorelease];
 }
 
 - (void) _createInternal
@@ -543,8 +563,9 @@ isEqualTree(xmlNodePtr nodeA, xmlNodePtr nodeB)
 
               if (ns->href == NULL)
                 {
-                  xmlNsPtr ns1 = xmlSearchNs(parentNode->doc, parentNode, ns->prefix);
+                  xmlNsPtr ns1;
 
+                  ns1 = xmlSearchNs(parentNode->doc, parentNode, ns->prefix);
                   if (ns1 != NULL)
                     {
                       cleanup_namespaces(childNode, ns1);
@@ -554,8 +575,10 @@ isEqualTree(xmlNodePtr nodeA, xmlNodePtr nodeB)
               /*
               else if (ns->prefix == NULL)
                 {
-                  xmlNsPtr ns1 = xmlSearchNsByHref(parentNode->doc, parentNode, ns->href);
+                  xmlNsPtr ns1;
 
+                  ns1
+                    = xmlSearchNsByHref(parentNode->doc, parentNode, ns->href);
                   if (ns1 != NULL)
                     {
                       cleanup_namespaces(childNode, ns1);
@@ -611,9 +634,9 @@ isEqualTree(xmlNodePtr nodeA, xmlNodePtr nodeB)
         }
     }
 
-  if (mergeTextNodes ||
-           ((childNode->type != XML_TEXT_NODE) &&
-            (parentNode->type != XML_TEXT_NODE)))
+  if (mergeTextNodes
+    || ((childNode->type != XML_TEXT_NODE)
+      && (parentNode->type != XML_TEXT_NODE)))
     {
       // this uses the built-in libxml functions which merge adjacent text nodes
       xmlNodePtr addedNode = NULL;
@@ -880,11 +903,11 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 {
   NSXMLNode *n;
 
-  n = [[[self alloc] initWithKind: NSXMLAttributeKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLAttributeKind];
   [n setStringValue: stringValue];
   [n setName: name];
   
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (id) attributeWithName: (NSString*)name
@@ -893,55 +916,55 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 {
   NSXMLNode *n;
   
-  n = [[[self alloc] initWithKind: NSXMLAttributeKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLAttributeKind];
   [n setURI: URI];
   [n setStringValue: stringValue];
   [n setName: name];
   
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (id) commentWithStringValue: (NSString*)stringValue
 {
   NSXMLNode *n;
 
-  n = [[[self alloc] initWithKind: NSXMLCommentKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLCommentKind];
   [n setStringValue: stringValue];
 
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (id) DTDNodeWithXMLString: (NSString*)string
 {
   NSXMLNode *n;
 
-  n = [[[NSXMLDTDNode alloc] initWithXMLString: string] autorelease];
+  n = [[NSXMLDTDNode alloc] initWithXMLString: string];
 
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (id) document
 {
   NSXMLNode *n;
 
-  n = [[[NSXMLDocument alloc] initWithKind: NSXMLDocumentKind] autorelease];
-  return n;
+  n = [[NSXMLDocument alloc] initWithKind: NSXMLDocumentKind];
+  return AUTORELEASE(n);
 }
 
 + (id) documentWithRootElement: (NSXMLElement*)element
 {
   NSXMLDocument	*d;
 
-  d = [[[NSXMLDocument alloc] initWithRootElement: element] autorelease];
-  return d;
+  d = [[NSXMLDocument alloc] initWithRootElement: element];
+  return AUTORELEASE(d);
 }
 
 + (id) elementWithName: (NSString*)name
 {
   NSXMLNode *n;
 
-  n = [[[NSXMLElement alloc] initWithName: name] autorelease];
-  return n;
+  n = [[NSXMLElement alloc] initWithName: name];
+  return AUTORELEASE(n);
 }
 
 + (id) elementWithName: (NSString*)name
@@ -960,8 +983,8 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 {
   NSXMLNode *n;
 
-  n = [[[NSXMLElement alloc] initWithName: name URI: URI] autorelease];
-  return n;
+  n = [[NSXMLElement alloc] initWithName: name URI: URI];
+  return AUTORELEASE(n);
 }
 
 + (id) elementWithName: (NSString*)name
@@ -970,7 +993,7 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
   NSXMLElement *e;
   
   e = [[NSXMLElement alloc] initWithName: name stringValue: string];
-  return e;
+  return AUTORELEASE(e);
 }
 
 + (NSString*) localNameForName: (NSString*)name
@@ -999,10 +1022,10 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 {
   NSXMLNode *n;
 
-  n = [[[self alloc] initWithKind: NSXMLNamespaceKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLNamespaceKind];
   [n setName: name];
   [n setStringValue: stringValue];
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (NSXMLNode*) predefinedNamespaceForPrefix: (NSString*)name
@@ -1065,19 +1088,19 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 {
   NSXMLNode *n;
 
-  n = [[[self alloc] initWithKind: NSXMLProcessingInstructionKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLProcessingInstructionKind];
   [n setStringValue: stringValue];
   [n setName: name];
-  return n;
+  return AUTORELEASE(n);
 }
 
 + (id) textWithStringValue: (NSString*)stringValue
 {
   NSXMLNode *n;
 
-  n = [[[self alloc] initWithKind: NSXMLTextKind] autorelease];
+  n = [[self alloc] initWithKind: NSXMLTextKind];
   [n setStringValue: stringValue];
-  return n;
+  return AUTORELEASE(n);
 }
 
 - (NSString*) canonicalXMLStringPreservingComments: (BOOL)comments
@@ -1198,10 +1221,10 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
         {
           [subNode detach];
         }
-      [theSubNodes release];
+      RELEASE(theSubNodes);
 
-      [internal->objectValue release];
-      [internal->subNodes release];
+      RELEASE(internal->objectValue);
+      RELEASE(internal->subNodes);
       if (theNode)
         {
           if (theNode->type == XML_NAMESPACE_DECL)
@@ -1366,7 +1389,7 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 	break;
 
       case NSXMLAttributeDeclarationKind: 
-	[self release];
+	RELEASE(self);
 	return nil;
 	
       default: 
@@ -1382,7 +1405,7 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
    */
   if (NO == [self isKindOfClass: theSubclass])
     {
-      [self release];
+      RELEASE(self);
       return [[theSubclass alloc] initWithKind: theKind
 				       options: theOptions];
     }
@@ -1865,7 +1888,7 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
                   [subNode detach];
                 }
             }
-          [theSubNodes release];
+          RELEASE(theSubNodes);
         }
 
       if (resolve == NO)
@@ -2092,4 +2115,277 @@ execute_xpath(xmlNodePtr node, NSString *xpath_exp, NSDictionary *constants,
 }
 @end
 
+#else
+
+#import "Foundation/NSException.h"
+#import "Foundation/NSXMLNode.h"
+
+@implementation NSXMLNode
+
++ (id) attributeWithName: (NSString*)name
+	     stringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
++ (id) attributeWithName: (NSString*)name
+		     URI: (NSString*)URI
+	     stringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
++ (id) commentWithStringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
++ (id) DTDNodeWithXMLString: (NSString*)string
+{
+  return nil;
+}
+
++ (id) document
+{
+  return nil;
+}
+
++ (id) documentWithRootElement: (NSXMLElement*)element
+{
+  return nil;
+}
+
++ (id) elementWithName: (NSString*)name
+{
+  return nil;
+}
+
++ (id) elementWithName: (NSString*)name
+	      children: (NSArray*)children
+	    attributes: (NSArray*)attributes
+{
+  return nil;
+}
+
++ (id) elementWithName: (NSString*)name
+		   URI: (NSString*)URI
+{
+  return nil;
+}
+
++ (id) elementWithName: (NSString*)name
+	   stringValue: (NSString*)string
+{
+  return nil;
+}
+
++ (NSString*) localNameForName: (NSString*)name
+{
+  return nil;
+}
+
++ (id) namespaceWithName: (NSString*)name
+	     stringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
++ (NSXMLNode*) predefinedNamespaceForPrefix: (NSString*)name
+{
+  return nil;
+}
+
++ (NSString*) prefixForName: (NSString*)name
+{
+  return nil;
+}
+
++ (id) processingInstructionWithName: (NSString*)name
+			 stringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
++ (id) textWithStringValue: (NSString*)stringValue
+{
+  return nil;
+}
+
+- (NSString*) canonicalXMLStringPreservingComments: (BOOL)comments
+{
+  return nil;
+}
+
+- (NSXMLNode*) childAtIndex: (NSUInteger)index
+{
+  return nil;
+}
+
+- (NSUInteger) childCount
+{
+  return 0;
+}
+
+- (NSArray*) children
+{
+  return nil;
+}
+
+- (id) copyWithZone: (NSZone*)zone
+{
+  return nil;
+}
+
+- (void) detach
+{
+}
+
+- (NSUInteger) index
+{
+  return 0;
+}
+
+- (id) init
+{
+  NSString      *className = NSStringFromClass([self class]);
+
+  DESTROY(self);
+  [NSException raise: NSGenericException
+              format: @"%@ - no libxml2 at configure time", className];
+  return nil;
+}
+
+- (id) initWithKind: (NSXMLNodeKind)theKind
+{
+  return [self init];
+}
+
+- (id) initWithKind: (NSXMLNodeKind)theKind options: (NSUInteger)theOptions
+{
+  return [self init];
+}
+
+- (NSXMLNodeKind) kind
+{
+  return 0;
+}
+
+- (NSUInteger) level
+{
+  return 0;
+}
+
+- (NSString*) localName
+{
+  return nil;
+}
+
+- (NSString*) name
+{
+  return nil;
+}
+
+- (NSXMLNode*) nextNode
+{
+  return nil;
+}
+
+- (NSXMLNode*) nextSibling
+{
+  return nil;
+}
+
+- (id) objectValue
+{
+  return nil;
+}
+
+- (NSXMLNode*) parent
+{
+  return nil;
+}
+
+- (NSString*) prefix
+{
+  return nil;
+}
+
+- (NSXMLNode*) previousNode
+{
+  return nil;
+}
+
+- (NSXMLNode*) previousSibling
+{
+  return nil;
+}
+
+- (NSXMLDocument*) rootDocument
+{
+  return nil;
+}
+
+- (NSString*) stringValue
+{
+  return nil;
+}
+
+- (void) setObjectValue: (id)value
+{
+}
+
+- (void) setName: (NSString *)name
+{
+}
+
+- (void) setStringValue: (NSString*)string
+{
+}
+
+- (void) setStringValue: (NSString*)string resolvingEntities: (BOOL)resolve
+{
+}
+
+- (void) setURI: (NSString*)URI
+{
+}
+
+- (NSString*) URI
+{
+  return nil;
+}
+
+- (NSString*) XMLString
+{
+  return nil;
+}
+
+- (NSString*) XMLStringWithOptions: (NSUInteger)theOptions
+{
+  return nil;
+}
+
+- (NSString*) XPath
+{
+  return nil;
+}
+
+- (NSArray*) nodesForXPath: (NSString*)anxpath error: (NSError**)error
+{
+  return nil;
+}
+
+ - (NSArray*) objectsForXQuery: (NSString*)xquery
+		     constants: (NSDictionary*)constants
+		         error: (NSError**)error
+{
+  return nil;
+}
+
+- (NSArray*) objectsForXQuery: (NSString*)xquery error: (NSError**)error
+{
+  return nil;
+}
+@end
+
 #endif	/* HAVE_LIBXML */
