@@ -285,7 +285,7 @@ static Class		messagePortClass = 0;
   [d handlePortMessage: m];
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return [PORT(self)->name hash];
 }
@@ -938,7 +938,7 @@ again:
  * the start can be written directly without having to copy data to another
  * buffer etc.
  */
-- (unsigned int) reservedSpaceLength
+- (NSUInteger) reservedSpaceLength
 {
   return sizeof(GSPortItemHeader) + sizeof(GSPortMsgHeader);
 }
@@ -969,7 +969,7 @@ again:
 		  msgid: (int)msgId
              components: (NSMutableArray*)components
                    from: (NSPort*)receivingPort
-               reserved: (unsigned)length
+               reserved: (NSUInteger)length
 {
   NSMutableData		*h = nil;
   NSMutableData		*first;
@@ -996,7 +996,7 @@ again:
       NSLog(@"Attempt to send through recv port");
     }
 
-  c = [components count];
+  c = (unsigned)[components count];
   if (c == 0)
     {
       NSLog(@"empty components sent");
