@@ -108,7 +108,7 @@ static NSString		*mainFont = nil;
   RELEASE(localRefs);
   RELEASE(projectRefs);
   RELEASE(indent);
-  [super dealloc];
+  DEALLOC
 }
 
 - (void) decIndent
@@ -593,9 +593,7 @@ static NSString		*mainFont = nil;
 
 - (void) outputNode: (GSXMLNode*)node to: (NSMutableString*)buf
 {
-#if GS_WITH_GC == 0
-  CREATE_AUTORELEASE_POOL(arp);
-#endif
+  ENTER_POOL
   GSXMLNode	*children = [node firstChild];
 
   if ([node type] == XML_ELEMENT_NODE)
@@ -1093,7 +1091,7 @@ static NSString		*mainFont = nil;
 	  if (base == nil)
 	    {
 	      NSLog(@"No 'base' document name supplied in gsdoc element");
-	      return;
+	      break;
 	    }
 	  nextFile = [prop objectForKey: @"next"];
 	  nextFile = [nextFile stringByAppendingPathExtension: @"html"];
@@ -1963,7 +1961,7 @@ static NSString		*mainFont = nil;
 	    }
 	}
     }
-  [arp drain];
+  LEAVE_POOL
 }
 
 /**
